{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvBreatheSkin.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvBreatheSkin;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Registry, inifiles, JvSlider, JvLabel, jpeg, Menus, JvPcx,
  JvImage, JvScrollingLabel, JvTypes;

type
  TJvBreatheLabel = class(TPersistent)
  private
    FLabel:TJvLabel;
    FLeft: Integer;
    FTop: Integer;
    FWidth: Integer;
    FHeight: Integer;
    FCaption: string;
    procedure SetCaption(const Value: string);
    procedure SetHeight(const Value: integer);
    procedure SetLeft(const Value: integer);
    procedure SetTop(const Value: integer);
    procedure SetWidth(const Value: integer);
    function GeTColor: TColor;
    function GetFont: TFont;
    function GetTransparent: boolean;
    procedure SeTColor(const Value: TColor);
    procedure SetFont(const Value: TFont);
    procedure SetTransparent(const Value: boolean);
    function GetHint: string;
    function GetShowHint: boolean;
    procedure SetHint(const Value: string);
    procedure SetShowHint(const Value: boolean);
    function GetPopupMenu: TPopupMenu;
    procedure SetPopupMenu(const Value: TPopupMenu);
  protected
  public
    constructor Create(AOwner: TComponent);
  published
    property Left:Integer read FLeft write SetLeft;
    property Top:Integer read FTop write SetTop;
    property Width:Integer read FWidth write SetWidth;
    property Height:Integer read FHeight write SetHeight;
    property Caption:string read FCaption write SetCaption;
    property Font:TFont read GetFont write SetFont;
    property Color:TColor read GeTColor write SeTColor;
    property Transparent:boolean read GetTransparent write SetTransparent;
    property Hint:string read GetHint write SetHint;
    property ShowHint:boolean read GetShowHint write SetShowHint;
    property PopupMenu:TPopupMenu read GetPopupMenu write SetPopupMenu;
  end;

  TJvBreatheScrollLabel = class(TPersistent)
  private
    FLabel:TJvScrollingLabel;
    FHeight: Integer;
    FTop: Integer;
    FWidth: Integer;
    FLeft: Integer;
    FCaption: string;
    function GeTColor: TColor;
    function GetDirection: TLabelDirection;
    function GetFont: TFont;
    function GetHint: string;
    function GetInterval: cardinal;
    function GetNoGrap: boolean;
    function GetScrolling: boolean;
    function GetShowHint: boolean;
    function GetTransparent: boolean;
    procedure SetCaption(const Value: string);
    procedure SeTColor(const Value: TColor);
    procedure SetDirection(const Value: TLabelDirection);
    procedure SetFont(const Value: TFont);
    procedure SetHeight(const Value: integer);
    procedure SetHint(const Value: string);
    procedure SetInterval(const Value: cardinal);
    procedure SetLeft(const Value: integer);
    procedure SetNoGrap(const Value: boolean);
    procedure SetScrolling(const Value: boolean);
    procedure SetShowHint(const Value: boolean);
    procedure SetTop(const Value: integer);
    procedure SetTransparent(const Value: boolean);
    procedure SetWidth(const Value: integer);
  protected
  public
    constructor Create(AOwner: TComponent);
  published
    property Left:Integer read FLeft write SetLeft;
    property Top:Integer read FTop write SetTop;
    property Width:Integer read FWidth write SetWidth;
    property Height:Integer read FHeight write SetHeight;
    property Caption:string read FCaption write SetCaption;
    property Font:TFont read GetFont write SetFont;
    property Color:TColor read GeTColor write SeTColor;
    property Transparent:boolean read GetTransparent write SetTransparent;
    property Hint:string read GetHint write SetHint;
    property ShowHint:boolean read GetShowHint write SetShowHint;
    property Scrolling:boolean read GetScrolling write SetScrolling;
    property ScrollDirectory:TLabelDirection read GetDirection write SetDirection;
    property ScrollInterval:cardinal read GetInterval write SetInterval;
    property NoGrap:boolean read GetNoGrap write SetNoGrap;
  end;

  TJvBreatheButton = class(TPersistent)
  private
    FButton:TJvImage;
    function GetEnabled: boolean;
    function GetHint: string;
    function GetShowHint: boolean;
    procedure SetEnabled(const Value: boolean);
    procedure SetHint(const Value: string);
    procedure SetShowHint(const Value: boolean);
    function GetMouseDown: TMouseEvent;
    function GetMouseMove: TMouseMoveEvent;
    function GetOnClick: TNotifyEvent;
    function GetOnDblClick: TNotifyEvent;
    function GetPopupMenu: TPopupMenu;
    function GetVisible: boolean;
    procedure SetMouseDown(const Value: TMouseEvent);
    procedure SetMouveMove(const Value: TMouseMoveEvent);
    procedure SetOnClick(const Value: TNotifyEvent);
    procedure SetOnDblClick(const Value: TNotifyEvent);
    procedure SetPopupMenu(const Value: TPopupMenu);
    procedure SetVisible(const Value: boolean);
    function GetMouseUp: TMouseEvent;
    procedure SetMouseUp(const Value: TMouseEvent);
  protected
  public
    constructor Create(AOwner: TComponent);
  published
    property Hint:string read GetHint write SetHint;
    property ShowHint:boolean read GetShowHint write SetShowHint;
    property Enabled:boolean read GetEnabled write SetEnabled;
    property PopupMenu:TPopupMenu read GetPopupMenu write SetPopupMenu;
    property Visible:boolean read GetVisible write SetVisible;
    property OnMouseMove:TMouseMoveEvent read GetMouseMove write SetMouveMove;
    property OnMouseDown:TMouseEvent read GetMouseDown write SetMouseDown;
    property OnMouseUp:TMouseEvent read GetMouseUp write SetMouseUp;
    property OnClick:TNotifyEvent read GetOnClick write SetOnClick;
    property OnDblClick:TNotifyEvent read GetOnDblClick write SetOnDblClick;
  end;

  TJvBreatheButtons = class(TPersistent)
  private
    FPlaylist: TJvBreatheButton;
    FPrev: TJvBreatheButton;
    FMove: TJvBreatheButton;
    FPlay: TJvBreatheButton;
    FNext: TJvBreatheButton;
    FExit: TJvBreatheButton;
    FOptions: TJvBreatheButton;
    FOpen: TJvBreatheButton;
    FPause: TJvBreatheButton;
    FId3: TJvBreatheButton;
    FForward: TJvBreatheButton;
    FRewind: TJvBreatheButton;
    FStop: TJvBreatheButton;
    FMinimize: TJvBreatheButton;
  protected
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy;override;
  published
    property Play:TJvBreatheButton read FPlay write FPlay;
    property Stop:TJvBreatheButton read FStop write FStop;
    property Pause:TJvBreatheButton read FPause write FPause;
    property Next:TJvBreatheButton read FNext write FNext;
    property Previous:TJvBreatheButton read FPrev write FPrev;
    property Open:TJvBreatheButton read FOpen write FOpen;
    property Exit:TJvBreatheButton read FExit write FExit;
    property Move:TJvBreatheButton read FMove write FMove;
    property Minimize:TJvBreatheButton read FMinimize write FMinimize;
    property Id3:TJvBreatheButton read FId3 write FId3;
    property Forward:TJvBreatheButton read FForward write FForward;
    property Rewind:TJvBreatheButton read FRewind write FRewind;
    property Options:TJvBreatheButton read FOptions write FOptions;
    property Playlist:TJvBreatheButton read FPlaylist write FPlaylist;
  end;

  TJvBreatheLabels = class(TPersistent)
  private
    FStatus: TJvBreatheLabel;
    FBitRate: TJvBreatheLabel;
    FTime: TJvBreatheLabel;
    FFrequency: TJvBreatheLabel;
    FSongName: TJvBreatheScrollLabel;
    FTotalInfo: TJvBreatheLabel;
    FTotalTime: TJvBreatheLabel;
    FLayer: TJvBreatheLabel;
    FVersion: TJvBreatheLabel;
  protected
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy;override;
  published
    property Status:TJvBreatheLabel read FStatus write FStatus;
    property CurrenTTime:TJvBreatheLabel read FTime write FTime;
    property Layer:TJvBreatheLabel read FLayer write FLayer;
    property BitRate:TJvBreatheLabel read FBitRate write FBitRate;
    property Frequency:TJvBreatheLabel read FFrequency write FFrequency;
    property Version:TJvBreatheLabel read FVersion write FVersion;
    property SongName:TJvBreatheScrollLabel read FSongName write FSongName;
    property TotalTime:TJvBreatheLabel read FTotalTime write FTotalTime;
    property TotalInfo:TJvBreatheLabel read FTotalInfo write FTotalInfo;
  end;

  TJvBreatheOption = class(TPersistent)
  private
    FAutoSize: boolean;
    FOnChange: TNotifyEvent;
    FAutoMove: boolean;
    FAutoRegion: boolean;
    FEasyMove: boolean;
    procedure SetAutoSize(const Value: boolean);
    procedure SetAutoMove(const Value: boolean);
    procedure SetAutoRegion(const Value: boolean);
    procedure SetEasyMove(const Value: boolean);
  protected
    property OnChange:TNotifyEvent read FOnChange write FOnChange;
  public
    constructor Create(AOwner: TComponent);
  published
    property AutoSize:boolean read FAutoSize write SetAutoSize default true;
    property AutoMove:boolean read FAutoMove write SetAutoMove default true;
    property AutoRegion:boolean read FAutoRegion write SetAutoRegion default false;
    property EasyMove:boolean read FEasyMove write SetEasyMove default false;
  end;

  TJvBreatheVolume = class(TPersistent)
  private
    FSlider:TJvSlider;
    function GetMaximum: Integer;
    function GetPosition: Integer;
    procedure SetMaximum(const Value: integer);
    procedure SetPosition(const Value: integer);
    function GetHint: string;
    function GetShow: boolean;
    procedure SetHint(const Value: string);
    procedure SetShow(const Value: boolean);
  protected
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy;override;
  published
    property Maximum:Integer read GetMaximum write SetMaximum;
    property Position:Integer read GetPosition write SetPosition;
    property Hint:string read GetHint write SetHint;
    property ShowHint:boolean read GetShow write SetShow;
  end;

  TJvBreathePosition = class(TPersistent)
  private
    FSlider:TJvSlider;
    function GetMaximum: Integer;
    function GetPosition: Integer;
    procedure SetMaximum(const Value: integer);
    procedure SetPosition(const Value: integer);
    function GetHint: string;
    procedure SetHint(const Value: string);
    function GetShow: boolean;
    procedure SetShow(const Value: boolean);
  protected
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy;override;
  published
    property Maximum:Integer read GetMaximum write SetMaximum;
    property Position:Integer read GetPosition write SetPosition;
    property Hint:string read GetHint write SetHint;
    property ShowHint:boolean read GetShow write SetShow;
  end;

  TJvBreatheSkin = class(TWinControl)
  private
    //Internal datas
    FImg:TImage;
    FSkin: TFileName;
    FBOptions: TJvBreatheOption;
    FLabels: TJvBreatheLabels;
    FTAbout:TstringList;
    FLastX,FLastY:Integer;

    //Sliders
    FVolume:TJvBreatheVolume;
    FPosition:TJvBreathePosition;

    //Images
    FBack:TBitmap;
    FDeact:TBitmap;
    FInfo:TBItmap;
    FMask:TBitmap;
    FOver:TBitmap;
    FPosRuler:TBitmap;
    FPosThumb:TBitmap;
    FVolRuler:TBitmap;
    FVolThumb:TBitmap;
    FOnMoveDown: TMouseEvent;
    FOnMoveMove: TMouseMoveEvent;
    FButtons: TJvBreatheButtons;
    FOnSkinLoaded: TNotifyEvent;

    //Methods
    procedure LoadDefault;
    procedure InitBtn(Value: TJvImage);
    procedure SetSkin(const Value: TFileName);
    procedure LoadSkin(Value:string);
    procedure InitAllBtn;
    procedure DecoupeOver(Value: TJvImage);
    procedure DecoupeOverAll;
    procedure LoadBtn(ini: TInifile; Value: TJvImage; prefix: string);
    procedure LoadSlider(ini: TInifile; Value: TJvSlider; prefix: string);
    procedure LoadLabel(ini: TInifile; Value: TJvBreatheLabel; prefix: string);overload;
    procedure LoadLabel(ini: TInifile; Value: TJvBreatheScrollLabel; prefix: string);overload;
    procedure OptionsChanged(Sender: TObject);
    function GetOnExit: TNotifyEvent;
    function GetOnForward: TNotifyEvent;
    function GetOnID3: TNotifyEvent;
    function GetOnMinimize: TNotifyEvent;
    function GetOnMove: TNotifyEvent;
    function GetOnNext: TNotifyEvent;
    function GetOnOpen: TNotifyEvent;
    function GetOnOptions: TNotifyEvent;
    function GetOnPause: TNotifyEvent;
    function GetOnPlay: TNotifyEvent;
    function GetOnPlaylist: TNotifyEvent;
    function GetOnPosition: TNotifyEvent;
    function GetOnPrevious: TNotifyEvent;
    function GetOnRewind: TNotifyEvent;
    function GetOnStop: TNotifyEvent;
    function GetOnVolume: TNotifyEvent;
    procedure SetOnExit(const Value: TNotifyEvent);
    procedure SetOnForward(const Value: TNotifyEvent);
    procedure SetOnID3(const Value: TNotifyEvent);
    procedure SetOnMinimize(const Value: TNotifyEvent);
    procedure SetOnMove(const Value: TNotifyEvent);
    procedure SetOnNext(const Value: TNotifyEvent);
    procedure SetOnOpen(const Value: TNotifyEvent);
    procedure SetOnOptions(const Value: TNotifyEvent);
    procedure SetOnPause(const Value: TNotifyEvent);
    procedure SetOnPlay(const Value: TNotifyEvent);
    procedure SetOnPlaylist(const Value: TNotifyEvent);
    procedure SetOnPosition(const Value: TNotifyEvent);
    procedure SetOnPrevious(const Value: TNotifyEvent);
    procedure SetOnRewind(const Value: TNotifyEvent);
    procedure SetOnStop(const Value: TNotifyEvent);
    procedure SetOnVolume(const Value: TNotifyEvent);
    function GetOnPosChanging: TNotifyEvent;
    function GetOnVolChanging: TNotifyEvent;
    procedure SetOnPosChanging(const Value: TNotifyEvent);
    procedure SetOnVolChanging(const Value: TNotifyEvent);
    function GetCurrenTTime: TNotifyEvent;
    function GetOnBitRate: TNotifyEvent;
    function GetOnFrequency: TNotifyEvent;
    function GetOnLayer: TNotifyEvent;
    function GetOnSongName: TNotifyEvent;
    function GetOnStatus: TNotifyEvent;
    function GetOnTotalInfo: TNotifyEvent;
    function GetOnTotalTime: TNotifyEvent;
    function GetOnVersion: TNotifyEvent;
    procedure SetCurrenTTime(const Value: TNotifyEvent);
    procedure SetOnBitRate(const Value: TNotifyEvent);
    procedure SetOnFrequency(const Value: TNotifyEvent);
    procedure SetOnLayer(const Value: TNotifyEvent);
    procedure SetOnSongName(const Value: TNotifyEvent);
    procedure SetOnStatus(const Value: TNotifyEvent);
    procedure SetOnTotalInfo(const Value: TNotifyEvent);
    procedure SetOnTotalTime(const Value: TNotifyEvent);
    procedure SetOnVersion(const Value: TNotifyEvent);
    procedure MoveFormDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: integer);
    procedure MoveFormMove(Sender: TObject; Shift: TShiftState; X, Y: integer);
    procedure EasyFormDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: integer);
    procedure EasyFormMove(Sender: TObject; Shift: TShiftState; X, Y: integer);
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property SkinFile:TFileName read FSkin write SetSkin;
    property Options:TJvBreatheOption read FBOptions write FBOptions;
    property Labels:TJvBreatheLabels read FLabels write FLabels;
    property Volume:TJvBreatheVolume read FVolume write FVolume;
    property Buttons:TJvBreatheButtons read FButtons write FButtons;
    property Position:TJvBreathePosition read FPosition write FPosition;
    property OnPlayClick:TNotifyEvent read GetOnPlay write SetOnPlay;
    property OnStopClick:TNotifyEvent read GetOnStop write SetOnStop;
    property OnPauseClick:TNotifyEvent read GetOnPause write SetOnPause;
    property OnNextClick:TNotifyEvent read GetOnNext write SetOnNext;
    property OnPreviousClick:TNotifyEvent read GetOnPrevious write SetOnPrevious;
    property OnForwardClick:TNotifyEvent read GetOnForward write SetOnForward;
    property OnRewindClick:TNotifyEvent read GetOnRewind write SetOnRewind;
    property OnOptionsClick:TNotifyEvent read GetOnOptions write SetOnOptions;
    property OnID3Click:TNotifyEvent read GetOnID3 write SetOnID3;
    property OnPlaylistClick:TNotifyEvent read GetOnPlaylist write SetOnPlaylist;
    property OnMinimizeClick:TNotifyEvent read GetOnMinimize write SetOnMinimize;
    property OnExitClick:TNotifyEvent read GetOnExit write SetOnExit;
    property OnMoveClick:TNotifyEvent read GetOnMove write SetOnMove;
    property OnMoveDown:TMouseEvent read FOnMoveDown write FOnMoveDown;
    property OnMoveMove:TMouseMoveEvent read FOnMoveMove write FOnMoveMove;
    property OnOpenClick:TNotifyEvent read GetOnOpen write SetOnOpen;
    property OnVolumeChanged:TNotifyEvent read GetOnVolume write SetOnVolume;
    property OnPositionChanged:TNotifyEvent read GetOnPosition write SetOnPosition;
    property OnVolumeChanging:TNotifyEvent read GetOnVolChanging write SetOnVolChanging;
    property OnPositionChanging:TNotifyEvent read GetOnPosChanging write SetOnPosChanging;
    property OnStatusClick:TNotifyEvent read GetOnStatus write SetOnStatus;
    property OnCurrenTTimeClick:TNotifyEvent read GetCurrenTTime write SetCurrenTTime;
    property OnLayerClick:TNotifyEvent read GetOnLayer write SetOnLayer;
    property OnBitRateClick:TNotifyEvent read GetOnBitRate write SetOnBitRate;
    property OnFrequencyClick:TNotifyEvent read GetOnFrequency write SetOnFrequency;
    property OnVersionClick:TNotifyEvent read GetOnVersion write SetOnVersion;
    property OnSongNameClick:TNotifyEvent read GetOnSongName write SetOnSongName;
    property OnTotalTimeClick:TNotifyEvent read GetOnTotalTime write SetOnTotalTime;
    property OnTotalInfoClick:TNotifyEvent read GetOnTotalInfo write SetOnTotalInfo;
    procedure ShowAbout;
    procedure Activate;
    procedure Deactivate;
    property PopupMenu;
    property OnSkinLoaded:TNotifyEvent read FOnSkinLoaded write FOnSkinLoaded;
  end;


implementation

{$R RES_Breathe.res}

resourcestring
   RC_AboutCaption     =    'About';

   RC_PrefixSongtitle  =    'songtitle';
   RC_PrefixTotalInfo  =    'totalinfo';
   RC_PrefixTotalTime  =    'totaltime';
   RC_PrefixTime       =    'time';
   RC_PrefixInfo       =    'info';
   RC_PrefixLayer      =    'layer';
   RC_PrefixBitRate    =    'BitRate';
   RC_PrefixFrequency  =    'frequency';
   RC_PrefixVersion    =    'versiontext';
   RC_PrefixVolbar     =    'volbar';
   RC_PrefixTbbar      =    'tbbar';
   RC_PrefixExit       =    'exit';
   RC_PrefixMove       =    'move';
   RC_PrefixMinimize   =    'mini';
   RC_PrefixOption     =    'option';
   RC_PrefixOpen       =    'open';
   RC_PrefixId3        =    'id3';
   RC_PrefixPlaylist   =    'plist';
   RC_PrefixNext       =    'next';
   RC_PrefixForward    =    'fwd';
   RC_PrefixPlay       =    'play';
   RC_PrefixPause      =    'pause';
   RC_PrefixStop       =    'stop';
   RC_PrefixRewind     =    'rev';
   RC_PrefixPrevious   =    'prev';

   RC_Section          =    'main';

   RC_SufixTransp      =    'transparent';
   RC_SufixTop         =    'top';
   RC_SufixLeft        =    'left';
   RC_SufixBKColor     =    'bkcolor';
   RC_SufixColor       =    'color';
   RC_SufixFontSize    =    'fontsize';
   RC_SufixFont        =    'font';
   RC_SufixHeight      =    'height';
   RC_SufixWidth       =    'width';
   RC_SufixVertical    =    'vertical';

   RC_InfoFile         =    'info.txt';

   RC_BmpThumb         =    'volthumb.bmp';
   RC_BmpVolRuler      =    'volruler.bmp';
   RC_BmpPosThumb      =    'posthumb.bmp';
   RC_BmpPosRuler      =    'posruler.bmp';
   RC_BmpOver          =    'over.bmp';
   RC_BmpMask          =    'mask.bmp';
   RC_BmpInfo          =    'info.bmp';
   RC_BmpDeact         =    'deact.bmp';
   RC_BmpBack          =    'back.bmp';

   RC_DefTotalInfo     =    'Info';
   RC_DefTotalTime     =    '00:00';
   RC_DefSongName      =    'none';
   RC_DefVersion       =    '1.0';
   RC_DefNothing       =    '-';
   RC_DefStatus        =    'Status';


{************************************************************}
procedure TJvBreatheSkin.LoadDefault;
var
   res:TResourceStream;
   buf:array[0..255] of char;
   st:string;

   procedure LoadFromRes(bmp:TBitmap;Value:string);
   var
     jpeg:TJpegImage;
     r:TResourceStream;
   begin
      r:=TResourceStream.Create(hInstance,Value,RT_RCDATA);
      jpeg:=TJPegImage.Create;
      jpeg.LoadFromStream(r);

      bmp.Assign(jpeg);

      r.Free;
      jpeg.free;
   end;

   procedure LoadFromResPcx(bmp:TBitmap;Value:string);
   var
     pcx:TJvPcx;
     r:TResourceStream;
   begin
      r:=TResourceStream.Create(hInstance,Value,RT_RCDATA);
      pcx:=TJvPcx.Create;
      pcx.LoadFromStream(r);

      bmp.Assign(pcx);

      r.Free;
      pcx.free;
   end;

begin
   //Load About text
   res:=TResourceStream.Create(hInstance,'infot',RT_RCDATA);
   FTAbout.LoadFromStream(res);
   res.free;

   LoadFromRes(FBack,'back');
   LoadFromRes(FDeact,'deact');
   LoadFromRes(FInfo,'info');
   LoadFromRes(FPosRuler,'posruler');
   LoadFromRes(FPosThumb,'posthumb');
   LoadFromRes(FVolRuler,'volruler');
   LoadFromRes(FVolThumb,'volthumb');

   LoadFromResPcx(FOver,'over');
   LoadFromResPcx(FMask,'mask');

   //assign back picture
   FImg.Picture.Assign(FBack);
   self.Width:=FImg.width;
   self.Height:=FImg.height;

   //load default skin
   GetTempPath(255,buf);
   st:=buf;
   if st[length(st)]<>'\' then st:=st+'\';

   res:=TResourceStream.Create(hInstance,'main',RT_RCDATA);
   res.SaveToFile(st+'1.ini');
   LoadSkin(st+'1.ini');
   DeleteFile(st+'1.ini');
end;
{************************************************************}
constructor TJvBreatheSkin.Create(AOwner: TComponent);
begin
   inherited;
   FSkin:='';
   FBOptions:=TJvBreatheOption.Create(self);
   FBOptions.OnChange:=OptionsChanged;
   FTAbout:=TstringList.Create;

   //enable painting ;)
   self.parent:=TWinControl(AOwner);

   FImg:=TImage.Create(self);
   FImg.AutoSize:=true;
   FImg.parent:=self;
   FImg.left:=0;
   FImg.top:=0;
   FImg.OnMouseDown:=EasyFormDown;
   FImg.OnMouseMove:=EasyFormMove;

   FLabels:=TJvBreatheLabels.Create(self);
   FButtons:=TJvBreatheButtons.Create(self);
   FButtons.Move.OnMouseMove:=MoveFormMove;
   FButtons.Move.OnMouseDown:=MoveFormDown;

   //Sliders
   FVolume:=TJvBreatheVolume.Create(self);
   FVolume.FSlider.left:=0;
   FVolume.FSlider.top:=0;
   FVolume.FSlider.width:=0;
   FVolume.FSlider.height:=0;
   FVolume.Maximum:=100;
   FVolume.Position:=100;

   FPosition:=TJvBreathePosition.Create(self);
   FPosition.FSlider.left:=0;
   FPosition.FSlider.top:=0;
   FPosition.FSlider.width:=0;
   FPosition.FSlider.height:=0;
   FPosition.FSlider.Parent:=self;

   //Images
   FBack:=TBitmap.Create;
   FDeact:=TBitmap.Create;
   FInfo:=TBitmap.Create;
   FMask:=TBitmap.Create;
   FOver:=TBitmap.Create;
   FPosRuler:=TBitmap.Create;
   FPosThumb:=TBitmap.Create;
   FVolRuler:=TBitmap.Create;
   FVolThumb:=TBitmap.Create;

   //Labels
   FLabels.Status.Caption:=RC_DefStatus;
   FLabels.CurrenTTime.Caption:=RC_DefTotalTime;
   FLabels.Layer.Caption:=RC_DefNothing;
   FLabels.BitRate.Caption:=RC_DefNothing;
   FLabels.Frequency.Caption:=RC_DefNothing;
   FLabels.Version.Caption:=RC_DefVersion;
   FLabels.SongName.Caption:=RC_DefSongName;
   FLabels.TotalTime.Caption:=RC_DefTotalTime;
   FLabels.TotalInfo.Caption:=RC_DefTotalInfo;

   //Load default design
   InitAllBtn;
   LoadDefault;
end;
{************************************************************}
destructor TJvBreatheSkin.Destroy;
var
   h:THandle;
begin
   try
     //Reset the region of the window if not destroying 
     if not(csDestroying in GetParentForm(TControl(self)).ComponentState) then
     begin
        //Enable caption
        h:=GetParenTForm(TControl(self)).Handle;
        SetWindowLong(h,GWL_style,GetWindowLong(h,GWL_style )or WS_CAPTION);

        //Use form region
        SetWindowRgn(h,0,true);
     end;
   except
   end;

   FBack.free;
   FDeact.free;
   FInfo.free;
   FMask.free;
   FOver.free;
   FPosRuler.free;
   FPosThumb.free;
   FVolRuler.free;
   FVolThumb.free;

   FImg.free;
   FVolume.free;
   FPosition.free;
   FBOptions.free;
   FLabels.free;
   FTAbout.free;
   FButtons.Free;

   inherited;
end;
{************************************************************}
procedure TJvBreatheSkin.InitBtn(Value:TJvImage);
begin
   Value.Left:=0;
   Value.top:=0;
   Value.width:=0;
   Value.height:=0;
end;
{************************************************************}
procedure TJvBreatheSkin.InitAllBtn;
begin
   InitBtn(FButtons.FPlay.FButton);
   InitBtn(FButtons.FStop.FButton);
   InitBtn(FButtons.FPause.FButton);
   InitBtn(FButtons.FNext.FButton);
   InitBtn(FButtons.FPrev.FButton);
   InitBtn(FButtons.FOpen.FButton);
   InitBtn(FButtons.FExit.FButton);
   InitBtn(FButtons.FMove.FButton);
   InitBtn(FButtons.FMinimize.FButton);
   InitBtn(FButtons.FId3.FButton);
   InitBtn(FButtons.FForward.FButton);
   InitBtn(FButtons.FRewind.FButton);
   InitBtn(FButtons.FOptions.FButton);
   InitBtn(FButtons.FPlaylist.FButton);
end;
{************************************************************}
procedure TJvBreatheSkin.SetSkin(const Value: TFileName);
var
   path:string;

   procedure LoadFromRes(bmp:TBitmap;Value:string);
   var
     jpeg:TJpegImage;
     r:TResourceStream;
   begin
      r:=TResourceStream.Create(hInstance,Value,RT_RCDATA);
      jpeg:=TJPegImage.Create;
      jpeg.LoadFromStream(r);

      bmp.Assign(jpeg);

      r.Free;
      jpeg.free;
   end;

begin
   FSkin := Value;
   if FSkin<>'' then
   begin
      //Load images
      path:=ExtractFilePath(Value);
      if fileexists(path+RC_BmpBack) then FBack.LoadFromFile(path+RC_BmpBack)
      else FBack.Assign(nil);
      if fileexists(path+RC_BmpDeact) then FDeact.LoadFromFile(path+RC_BmpDeact)
      else FDeact.Assign(nil);
      if fileexists(path+RC_BmpInfo) then FInfo.LoadFromFile(path+RC_BmpInfo)
      else FInfo.Assign(nil);
      if fileexists(path+RC_BmpMask) then FMask.LoadFromFile(path+RC_BmpMask)
      else FMask.Assign(nil);
      if fileexists(path+RC_BmpOver) then FOver.LoadFromFile(path+RC_BmpOver)
      else FOver.Assign(nil);

      if fileexists(path+RC_BmpPosRuler) then FPosRuler.LoadFromFile(path+RC_BmpPosRuler)
      else LoadFromRes(FPosRuler,'posruler');
      if fileexists(path+RC_BmpPosThumb) then FPosThumb.LoadFromFile(path+RC_BmpPosThumb)
      else LoadFromRes(FPosThumb,'posthumb');
      if fileexists(path+RC_BmpVolRuler) then FVolRuler.LoadFromFile(path+RC_BmpVolRuler)
      else LoadFromRes(FVolRuler,'volruler');
      if fileexists(path+RC_BmpThumb) then FVolThumb.LoadFromFile(path+RC_BmpThumb)
      else LoadFromRes(FVolThumb,'volthumb');

      //Assign background
      FImg.Picture.Bitmap.Assign(FBack);

      if fileexists(path+RC_InfoFile) then FTAbout.LoadFromFile(path+RC_InfoFile)
      else FTAbout.Clear;

      //if AutoSize
      if FBOptions.AutoSize then
      begin
         self.width:=FImg.width;
         self.height:=FImg.height;
      end;

      //Load ini file
      LoadSkin(FSkin);
   end
   else
     LoadDefault;
end;
{************************************************************}
procedure TJvBreatheSkin.DecoupeOver(Value:TJvImage);
var
   src:Trect;
   bmp:TBItmap;
begin
   src.left:=Value.Left;
   src.top:=Value.top;
   src.right:=Value.left+Value.width;
   src.Bottom:=Value.top+Value.height;

   bmp:=TBItmap.Create;
   bmp.width:=Value.width;
   bmp.height:=Value.height;
   bmp.Canvas.CopyRect(rect(0,0,Value.width,Value.height),FOver.Canvas,src);

   Value.Picture.Bitmap.Assign(bmp);

   bmp.Free;
end;
{************************************************************}
procedure TJvBreatheSkin.DecoupeOverAll;
begin
   DecoupeOver(FButtons.FPlay.FButton);
   DecoupeOver(FButtons.FStop.FButton);
   DecoupeOver(FButtons.FPause.FButton);
   DecoupeOver(FButtons.FNext.FButton);
   DecoupeOver(FButtons.FPrev.FButton);
   DecoupeOver(FButtons.FOpen.FButton);
   DecoupeOver(FButtons.FExit.FButton);
   DecoupeOver(FButtons.FMove.FButton);
   DecoupeOver(FButtons.FMinimize.FButton);
   DecoupeOver(FButtons.FId3.FButton);
   DecoupeOver(FButtons.FForward.FButton);
   DecoupeOver(FButtons.FRewind.FButton);
   DecoupeOver(FButtons.FOptions.FButton);
   DecoupeOver(FButtons.FPlaylist.FButton);
end;
{************************************************************}
procedure TJvBreatheSkin.LoadBtn(ini:TInifile;Value:TJvImage;prefix:string);
begin
   Value.Top:=ini.ReadInteger(RC_Section,prefix+RC_SufixTop,0);
   Value.Left:=ini.ReadInteger(RC_Section,prefix+RC_SufixLeft,0);
   Value.width:=ini.ReadInteger(RC_Section,prefix+RC_SufixWidth,0);
   Value.height:=ini.ReadInteger(RC_Section,prefix+RC_SufixHeight,0);
end;
{************************************************************}
procedure TJvBreatheSkin.LoadSlider(ini:TInifile;Value:TJvSlider;prefix:string);
begin
   Value.Horizontal:=not(ini.ReadBool(RC_Section,prefix+RC_SufixVertical,false));
   Value.Top:=ini.ReadInteger(RC_Section,prefix+RC_SufixTop,0);
   Value.Left:=ini.ReadInteger(RC_Section,prefix+RC_SufixLeft,0);
   Value.width:=ini.ReadInteger(RC_Section,prefix+RC_SufixWidth,0);
   Value.height:=ini.ReadInteger(RC_Section,prefix+RC_SufixHeight,0);
end;
{************************************************************}
procedure TJvBreatheSkin.LoadLabel(ini:TInifile;Value:TJvBreatheLabel;prefix:string);
var
  i:Integer;
begin
   Value.transparent:=ini.ReadBool(RC_Section,prefix+RC_SufixTransp,false);
   Value.Top:=ini.ReadInteger(RC_Section,prefix+RC_SufixTop,0);
   Value.Left:=ini.ReadInteger(RC_Section,prefix+RC_SufixLeft,0);
   Value.width:=ini.ReadInteger(RC_Section,prefix+RC_SufixWidth,0);
   Value.height:=ini.ReadInteger(RC_Section,prefix+RC_SufixHeight,0);
   Value.Font.Name:=ini.ReadString(RC_Section,prefix+RC_SufixFont,'arial');

   i:=ini.ReadInteger(RC_Section,prefix+RC_SufixFontSize,8);
   with TControlCanvas.Create do
   begin
      Control:=Value.FLabel;
      font.Name:=Value.font.name;
      font.Size:=i;
      if Value.height>0 then
        while TextHeight('abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ+-*/|')>Value.height do
        begin
           dec(i);
           font.size:=i;
        end;
      free;
   end;
   Value.Font.Size:=i;
   Value.Font.color:=ini.ReadInteger(RC_Section,prefix+RC_SufixColor,clblack);
   Value.color:=ini.ReadInteger(RC_Section,prefix+RC_SufixBKColor,clblack);
end;
{************************************************************}
procedure TJvBreatheSkin.LoadLabel(ini:TInifile;Value:TJvBreatheScrollLabel;prefix:string);
var
  i:Integer;
begin
   Value.transparent:=ini.ReadBool(RC_Section,prefix+RC_SufixTransp,false);
   Value.Top:=ini.ReadInteger(RC_Section,prefix+RC_SufixTop,0);
   Value.Left:=ini.ReadInteger(RC_Section,prefix+RC_SufixLeft,0);
   Value.width:=ini.ReadInteger(RC_Section,prefix+RC_SufixWidth,0);
   Value.height:=ini.ReadInteger(RC_Section,prefix+RC_SufixHeight,0);
   Value.Font.Name:=ini.ReadString(RC_Section,prefix+RC_SufixFont,'arial');

   i:=ini.ReadInteger(RC_Section,prefix+RC_SufixFontSize,8);
   with TControlCanvas.Create do
   begin
      Control:=Value.FLabel;
      font.Name:=Value.font.name;
      font.Size:=i;
      if Value.height>0 then
        while TextHeight('abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ+-*/|')>Value.height do
        begin
           dec(i);
           font.size:=i;
        end;
      free;
   end;
   Value.Font.Size:=i;
   Value.Font.color:=ini.ReadInteger(RC_Section,prefix+RC_SufixColor,clblack);
   Value.color:=ini.ReadInteger(RC_Section,prefix+RC_SufixBKColor,clblack);
end;
{************************************************************}
procedure TJvBreatheSkin.LoadSkin(Value: string);
var
   ini:TInifile;
begin
   ini:=TIniFile.Create(Value);

   InitAllBtn;

   LoadBtn(ini,FButtons.FPrev.FButton,RC_PrefixPrevious);
   LoadBtn(ini,FButtons.FRewind.FButton,RC_PrefixRewind);
   LoadBtn(ini,FButtons.FStop.FButton,RC_PrefixStop);
   LoadBtn(ini,FButtons.FPause.FButton,RC_PrefixPause);
   LoadBtn(ini,FButtons.FPlay.FButton,RC_PrefixPlay);
   LoadBtn(ini,FButtons.FForward.FButton,RC_PrefixForward);
   LoadBtn(ini,FButtons.FNext.FButton,RC_PrefixNext);
   LoadBtn(ini,FButtons.FPlaylist.FButton,RC_PrefixPlaylist);
   LoadBtn(ini,FButtons.FId3.FButton,RC_PrefixId3);
   LoadBtn(ini,FButtons.FOpen.FButton,RC_PrefixOpen);
   LoadBtn(ini,FButtons.FOptions.FButton,RC_PrefixOption);
   LoadBtn(ini,FButtons.FMinimize.FButton,RC_PrefixMinimize);
   LoadBtn(ini,FButtons.FMove.FButton,RC_PrefixMove);
   LoadBtn(ini,FButtons.FExit.FButton,RC_PrefixExit);

   FPosition.FSlider.ImageRuler.Assign(FPosRuler);
   FPosition.FSlider.ImageThumb.Assign(FPosThumb);
   FVolume.FSlider.ImageRuler.Assign(FVolRuler);
   FVolume.FSlider.ImageThumb.Assign(FVolThumb);
   LoadSlider(ini,FPosition.FSlider,RC_PrefixTbbar);
   LoadSlider(ini,FVolume.FSlider,RC_PrefixVolbar);

   LoadLabel(ini,FLabels.Version,RC_PrefixVersion);
   LoadLabel(ini,FLabels.Frequency,RC_PrefixFrequency);
   LoadLabel(ini,FLabels.BitRate,RC_PrefixBitRate);
   LoadLabel(ini,FLabels.Layer,RC_PrefixLayer);
   LoadLabel(ini,FLabels.Status,RC_PrefixInfo);
   LoadLabel(ini,FLabels.CurrenTTime,RC_PrefixTime);
   LoadLabel(ini,FLabels.TotalTime,RC_PrefixTotalTime);
   LoadLabel(ini,FLabels.TotalInfo,RC_PrefixTotalInfo);
   LoadLabel(ini,FLabels.SongName,RC_PrefixSongtitle);

   if not(FOver.Empty) then
        DecoupeOverAll;

   ini.free;

   OptionsChanged(self);
   if Assigned(FOnSkinLoaded) then
     FOnSkinLoaded(self);
end;
{*******************************************************}
function RegionFromBitmap(image:TBitmap):hrgn;
var
   rgn1,rgn2:hrgn;
   startx,endx,x,y:Integer;
   TransParentColor:TRGBTriple;
   bmp:TBitmap;
   p:pRGBArray;
begin
   rgn1 := 0;

   bmp:=TBitmap.Create;
   bmp.Assign(image);
   bmp.PixelFormat:=pf24Bit;

   if (bmp.height>0)and(bmp.width>0) then
   begin
      p:=bmp.ScanLine[0];
      TransParentColor:=p[0];
   end;

   for y:=0 to bmp.Height-1 do
   begin
      x:=0;
      p:=bmp.ScanLine[y];
      repeat
          while (x<bmp.width) and (CompareMem(@p[x],@TransParentColor,3)) do inc(x);
          inc(x);
          startx:=x;
          while (x<bmp.width) and (not(CompareMem(@p[x],@TransParentColor,3))) do inc(x);
          endx:=x;

          // do we have some pixels?
          if startx<bmp.Width then
          begin
             if rgn1 = 0 then
                 // Create a region to start with
                 rgn1:=CreateRectRgn(startx+1,y,endx,y+1)
             else
             begin
                rgn2:=CreateRectRgn(startx+1,y,endx,y+1);
                if rgn2<>0 then CombineRgn(rgn1,rgn1,rgn2,RGN_OR);
                DeleteObject(rgn2);
             end;
          end;
      until x>=image.width;
   end;
   
   bmp.free;
   result:=rgn1;
end;
{************************************************************}
procedure TJvBreatheSkin.OptionsChanged(Sender: TObject);
var
   h:THandle;
begin
   if FBOptions.AutoSize then
   begin
      self.Width := FBack.Width;
      self.height := FBack.height;
   end;
   if FBOptions.AutoRegion then
   begin
      self.left:=-3;
      self.top:=-3;

      //Remove caption
      h:=GetParenTForm(TControl(self)).Handle;
      SetWindowLong(h,GWL_style,GetWindowLong(h,GWL_style )and not WS_CAPTION);
      GetParenTForm(TControl(self)).ClientWidth:=self.width;
      GetParenTForm(TControl(self)).ClientHeight:=self.height;
      GetParenTForm(TControl(self)).Invalidate;

      //Use region
      SetWindowRgn(h,RegionFromBitmap(FMask),true);
   end
   else
   begin
      //Enable caption
      h:=GetParenTForm(TControl(self)).Handle;
      SetWindowLong(h,GWL_style,GetWindowLong(h,GWL_style )or WS_CAPTION);
      GetParenTForm(TControl(self)).Invalidate;

      //Use form region
      SetWindowRgn(h,0,true);
   end;
end;
{************************************************************}
procedure TJvBreatheSkin.ShowAbout;
var
   im:TJvImage;
   t:TForm;
   m:TMemo;
begin
   if not(FInfo.Empty)or(FTAbout.count>0) then
   begin
      t:=TForm.Create(Application);
      with t do
      begin
         caption:=RC_AboutCaption;
         if FTAbout.count>0 then width:=500
         else ClientWidth:=137;
         ClientHeight:=157;
         Borderstyle:=bsDialog;
         Position:=poScreenCenter;

         im:=TJvImage.Create(t);
         if not(FInfo.Empty) then
         begin
            im.parent:=t;
            im.top:=0;
            im.left:=0;
            im.width:=137;
            im.AutoSize:=false;
            im.Stretch:=true;
            im.Align:=alLeft;
            im.Picture.Bitmap.Assign(FInfo);
            im.Pictures.PicEnter.Bitmap.Assign(FInfo);
         end;

         m:=TMemo.Create(t);
         if FTAbout.count>0 then
         begin
            m.parent:=t;
            m.Align:=alClient;
            m.ReadOnly:=true;
            m.ScrollBars:=ssBoth;
            m.Text:=FTAbout.Text;
         end;

         showmodal;

         m.free;
         im.free;

         free;
      end;
   end;
end;
{************************************************************}
procedure TJvBreatheSkin.MoveFormDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
   FLastX:=x;
   FLastY:=y;

   if Assigned(FOnMoveDown) then
      FOnMoveDown(Sender,Button,Shift,X,Y);
end;
{************************************************************}
procedure TJvBreatheSkin.MoveFormMove(Sender: TObject; Shift: TShiftState; X,
  Y: integer);
begin
   if (FBOptions.AutoMove) and (HiWord (GetAsyncKeystate(VK_LBUTTON))>0) then
   begin
      GetParenTForm(self).Left:=GetParenTForm(self).Left+x-FLastX;
      GetParenTForm(self).Top:=GetParenTForm(self).Top+y-FLastY;
   end;

   if Assigned(FOnMoveMove) then
      FOnMoveMove(Sender,Shift,X,Y);
end;
{************************************************************}
procedure TJvBreatheSkin.EasyFormDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
   FLastX:=x;
   FLastY:=y;
end;
{************************************************************}
procedure TJvBreatheSkin.EasyFormMove(Sender: TObject; Shift: TShiftState;
  X, Y: integer);
begin
   if (FBOptions.AutoMove) and (HiWord (GetAsyncKeystate(VK_LBUTTON))>0) then
   begin
      GetParenTForm(self).Left:=GetParenTForm(self).Left+x-FLastX;
      GetParenTForm(self).Top:=GetParenTForm(self).Top+y-FLastY;
   end;
end;
{************************************************************}
procedure TJvBreatheSkin.Activate;
begin
   FImg.Picture.Bitmap.Assign(FBack);
end;
{************************************************************}
procedure TJvBreatheSkin.Deactivate;
begin
   if not(FDeact.empty) then
      FImg.Picture.Bitmap.Assign(FDeact);
end;
{************************************************************}
function TJvBreatheSkin.GetOnExit: TNotifyEvent;
begin
   result:=FButtons.FExit.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnForward: TNotifyEvent;
begin
   result:=FButtons.FForward.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnID3: TNotifyEvent;
begin
   result:=FButtons.FID3.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnMinimize: TNotifyEvent;
begin
   result:=FButtons.FMinimize.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnMove: TNotifyEvent;
begin
  result:=FButtons.FMove.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnNext: TNotifyEvent;
begin
   result:=FButtons.FNext.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnOpen: TNotifyEvent;
begin
   result:=FButtons.FOpen.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnOptions: TNotifyEvent;
begin
   result:=FButtons.FOptions.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnPause: TNotifyEvent;
begin
   result:=FButtons.FPause.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnPlay: TNotifyEvent;
begin
   result:=FButtons.FPlay.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnPlaylist: TNotifyEvent;
begin
   result:=FButtons.FPlaylist.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnPosition: TNotifyEvent;
begin
   result:=FPosition.FSlider.OnStopChanged;
end;
{************************************************************}
function TJvBreatheSkin.GetOnPrevious: TNotifyEvent;
begin
   result:=FButtons.FPrev.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnRewind: TNotifyEvent;
begin
   result:=FButtons.FRewind.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnStop: TNotifyEvent;
begin
   result:=FButtons.FStop.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnVolume: TNotifyEvent;
begin
   result:=FVolume.FSlider.OnStopChanged;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnExit(const Value: TNotifyEvent);
begin
   FButtons.FExit.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnForward(const Value: TNotifyEvent);
begin
   FButtons.FForward.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnID3(const Value: TNotifyEvent);
begin
   FButtons.FID3.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnMinimize(const Value: TNotifyEvent);
begin
   FButtons.FMinimize.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnMove(const Value: TNotifyEvent);
begin
   FButtons.FMove.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnNext(const Value: TNotifyEvent);
begin
   FButtons.FNext.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnOpen(const Value: TNotifyEvent);
begin
   FButtons.FOpen.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnOptions(const Value: TNotifyEvent);
begin
   FButtons.FOptions.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnPause(const Value: TNotifyEvent);
begin
   FButtons.FPause.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnPlay(const Value: TNotifyEvent);
begin
   FButtons.FPlay.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnPlaylist(const Value: TNotifyEvent);
begin
   FButtons.FPlayList.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnPosition(const Value: TNotifyEvent);
begin
   FPosition.FSlider.OnStopChanged:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnPrevious(const Value: TNotifyEvent);
begin
   FButtons.FPrev.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnRewind(const Value: TNotifyEvent);
begin
   FButtons.FRewind.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnStop(const Value: TNotifyEvent);
begin
   FButtons.FStop.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnVolume(const Value: TNotifyEvent);
begin
   FVolume.FSlider.OnStopChanged:=Value;
end;
{************************************************************}
function TJvBreatheSkin.GetOnPosChanging: TNotifyEvent;
begin
   result:=FPosition.FSlider.OnChanged;
end;
{************************************************************}
function TJvBreatheSkin.GetOnVolChanging: TNotifyEvent;
begin
   result:=FVolume.FSlider.OnChanged;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnPosChanging(const Value: TNotifyEvent);
begin
   FPosition.FSlider.OnChanged:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnVolChanging(const Value: TNotifyEvent);
begin
   FVolume.FSlider.OnChanged:=Value;
end;
{************************************************************}
function TJvBreatheSkin.GetCurrenTTime: TNotifyEvent;
begin
   result:=FLabels.FTime.FLabel.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnBitRate: TNotifyEvent;
begin
   result:=FLabels.FBitRate.FLabel.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnFrequency: TNotifyEvent;
begin
   result:=FLabels.FFrequency.FLabel.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnLayer: TNotifyEvent;
begin
   result:=FLabels.FLayer.FLabel.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnSongName: TNotifyEvent;
begin
   result:=FLabels.FSongName.FLabel.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnStatus: TNotifyEvent;
begin
   result:=FLabels.FStatus.FLabel.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnTotalInfo: TNotifyEvent;
begin
   result:=FLabels.FTotalInfo.FLabel.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnTotalTime: TNotifyEvent;
begin
   result:=FLabels.FTotalTime.FLabel.OnClick;
end;
{************************************************************}
function TJvBreatheSkin.GetOnVersion: TNotifyEvent;
begin
   result:=FLabels.FVersion.FLabel.OnClick;
end;
{************************************************************}
procedure TJvBreatheSkin.SetCurrenTTime(const Value: TNotifyEvent);
begin
   FLabels.FTime.FLabel.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnBitRate(const Value: TNotifyEvent);
begin
   FLabels.FBitRate.FLabel.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnFrequency(const Value: TNotifyEvent);
begin
   FLabels.FFrequency.FLabel.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnLayer(const Value: TNotifyEvent);
begin
   FLabels.FLayer.FLabel.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnSongName(const Value: TNotifyEvent);
begin
   FLabels.FSongName.FLabel.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnStatus(const Value: TNotifyEvent);
begin
   FLabels.FStatus.FLabel.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnTotalInfo(const Value: TNotifyEvent);
begin
   FLabels.FTotalInfo.FLabel.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnTotalTime(const Value: TNotifyEvent);
begin
   FLabels.FTotalTime.FLabel.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheSkin.SetOnVersion(const Value: TNotifyEvent);
begin
   FLabels.FVersion.FLabel.OnClick:=Value;
end;
{************************************************************}

///////////////////////////////////////////////////////////
// TJvBreatheOption
///////////////////////////////////////////////////////////

{************************************************************}
constructor TJvBreatheOption.Create(AOwner: TComponent);
begin
   FAutoSize:=true;
   FAutoMove:=true;
   FAutoRegion:=false;
   FEasyMove:=false;
end;
{************************************************************}
procedure TJvBreatheOption.SetAutoMove(const Value: boolean);
begin
   FAutoMove := Value;
   if Assigned(FOnChange) then
     FOnChange(self);
end;
{************************************************************}
procedure TJvBreatheOption.SetAutoRegion(const Value: boolean);
begin
   FAutoRegion := Value;
   if Assigned(FOnChange) then
     FOnChange(self);
end;
{************************************************************}
procedure TJvBreatheOption.SetAutoSize(const Value: boolean);
begin
   FAutoSize := Value;
   if Assigned(FOnChange) then
     FOnChange(self);
end;
{************************************************************}
procedure TJvBreatheOption.SetEasyMove(const Value: boolean);
begin
   FEasyMove := Value;
   if Assigned(FOnChange) then
     FOnChange(self);
end;
{************************************************************}

///////////////////////////////////////////////////////////
// TJvBreatheLabels
///////////////////////////////////////////////////////////

{************************************************************}
constructor TJvBreatheLabels.Create(AOwner: TComponent);
begin
   FStatus:=TJvBreatheLabel.Create(AOwner);
   FBitRate:=TJvBreatheLabel.Create(AOwner);
   FTime:=TJvBreatheLabel.Create(AOwner);
   FFrequency:=TJvBreatheLabel.Create(AOwner);
   FSongName:=TJvBreatheScrollLabel.Create(AOwner);
   FTotalInfo:=TJvBreatheLabel.Create(AOwner);
   FTotalTime:=TJvBreatheLabel.Create(AOwner);
   FLayer:=TJvBreatheLabel.Create(AOwner);
   FVersion:=TJvBreatheLabel.Create(AOwner);
end;
{************************************************************}
destructor TJvBreatheLabels.Destroy;
begin
   FStatus.free;
   FBitRate.free;
   FTime.free;
   FFrequency.free;
   FSongName.free;
   FTotalInfo.free;
   FTotalTime.free;
   FLayer.free;
   FVersion.free;
   inherited;
end;
{************************************************************}

///////////////////////////////////////////////////////////
// TJvBreatheLabel
///////////////////////////////////////////////////////////

{************************************************************}
constructor TJvBreatheLabel.Create(AOwner: TComponent);
begin
   FLabel:=TJvLabel.Create(AOwner);
   FLabel.Parent:=TwinControl(AOwner);
   FLabel.AutoSize:=false;
   FLabel.HotTrack:=false;
   FLabel.ParentColor:=false;
   FLabel.ParentFont:=false;
   FLabel.ParentShowHint:=false;
end;
{************************************************************}
function TJvBreatheLabel.GeTColor: TColor;
begin
   result:=FLabel.color;
end;
{************************************************************}
function TJvBreatheLabel.GetFont: TFont;
begin
   result:=FLabel.font;
end;
{************************************************************}
function TJvBreatheLabel.GetHint: string;
begin
   result:=FLabel.Hint;
end;
{************************************************************}
function TJvBreatheLabel.GetPopupMenu: TPopupMenu;
begin
   result:=FLabel.PopupMenu;
end;
{************************************************************}
function TJvBreatheLabel.GetShowHint: boolean;
begin
   result:=Flabel.ShowHint;
end;
{************************************************************}
function TJvBreatheLabel.GetTransparent: boolean;
begin
   result:=FLabel.Transparent;
end;
{************************************************************}
procedure TJvBreatheLabel.SetCaption(const Value: string);
begin
   FCaption := Value;
   FLabel.caption:=Value;
end;
{************************************************************}
procedure TJvBreatheLabel.SeTColor(const Value: TColor);
begin
   FLabel.Color:=Value;
end;
{************************************************************}
procedure TJvBreatheLabel.SetFont(const Value: TFont);
begin
   FLabel.Font.Assign(Value);
end;
{************************************************************}
procedure TJvBreatheLabel.SetHeight(const Value: integer);
begin
   FHeight := Value;
   FLabel.height:=Value;
end;
{************************************************************}
procedure TJvBreatheLabel.SetHint(const Value: string);
begin
   FLabel.hint:=Value;
end;
{************************************************************}
procedure TJvBreatheLabel.SetLeft(const Value: integer);
begin
   FLeft := Value;
   FLabel.left:=Value;
end;
{************************************************************}
procedure TJvBreatheLabel.SetPopupMenu(const Value: TPopupMenu);
begin
   FLabel.PopupMenu:=Value;
end;
{************************************************************}
procedure TJvBreatheLabel.SetShowHint(const Value: boolean);
begin
   FLabel.ShowHint:=Value;
end;
{************************************************************}
procedure TJvBreatheLabel.SetTop(const Value: integer);
begin
   FTop := Value;
   FLabel.top:=Value;
end;
{************************************************************}
procedure TJvBreatheLabel.SetTransparent(const Value: boolean);
begin
   FLabel.Transparent:=Value;
end;
{************************************************************}
procedure TJvBreatheLabel.SetWidth(const Value: integer);
begin
   FWidth := Value;
   FLabel.width:=Value;
end;
{************************************************************}

///////////////////////////////////////////////////////////
// TJvBreatheVolume
///////////////////////////////////////////////////////////

{************************************************************}
constructor TJvBreatheVolume.Create(AOwner: TComponent);
begin
   FSlider:=TJvSlider.Create(AOwner);
   FSlider.parent:=TWinControl(AOwner);
end;
{************************************************************}
destructor TJvBreatheVolume.Destroy;
begin
   FSlider.free;
   inherited;
end;
{************************************************************}
function TJvBreatheVolume.GetHint: string;
begin
   result:=FSlider.Hint;
end;
{************************************************************}
function TJvBreatheVolume.GetMaximum: Integer;
begin
   result:=FSlider.Maximum;
end;
{************************************************************}
function TJvBreatheVolume.GetPosition: Integer;
begin
   result:=FSlider.Position;
end;
{************************************************************}
function TJvBreatheVolume.GetShow: boolean;
begin
   result:=FSlider.ShowHint;  
end;
{************************************************************}
procedure TJvBreatheVolume.SetHint(const Value: string);
begin
   FSlider.Hint:=Value;
end;
{************************************************************}
procedure TJvBreatheVolume.SetMaximum(const Value: integer);
begin
   FSlider.Maximum:=Value;
end;
{************************************************************}
procedure TJvBreatheVolume.SetPosition(const Value: integer);
begin
   FSlider.Position:=Value;
end;
{************************************************************}
procedure TJvBreatheVolume.SetShow(const Value: boolean);
begin
   FSlider.ShowHint:=Value;
end;
{************************************************************}

///////////////////////////////////////////////////////////
// TJvBreathePosition
///////////////////////////////////////////////////////////

{************************************************************}
constructor TJvBreathePosition.Create(AOwner: TComponent);
begin
   FSlider:=TJvSlider.Create(AOwner);
   FSlider.parent:=TWinControl(AOwner);
end;
{************************************************************}
destructor TJvBreathePosition.Destroy;
begin
   FSlider.free;
   inherited;
end;
{************************************************************}
function TJvBreathePosition.GetHint: string;
begin
   result:=FSlider.Hint;
end;
{************************************************************}
function TJvBreathePosition.GetMaximum: Integer;
begin
   result:=FSlider.Maximum;
end;
{************************************************************}
function TJvBreathePosition.GetPosition: Integer;
begin
   result:=FSlider.Position;
end;
{************************************************************}
function TJvBreathePosition.GetShow: boolean;
begin
   result:=FSlider.ShowHint;
end;
{************************************************************}
procedure TJvBreathePosition.SetHint(const Value: string);
begin
   FSlider.Hint:=Value;
end;
{************************************************************}
procedure TJvBreathePosition.SetMaximum(const Value: integer);
begin
   FSlider.Maximum:=Value;
end;
{************************************************************}
procedure TJvBreathePosition.SetPosition(const Value: integer);
begin
   FSlider.Position:=Value;
end;
{************************************************************}
procedure TJvBreathePosition.SetShow(const Value: boolean);
begin
   FSlider.ShowHint:=Value;
end;
{************************************************************}

///////////////////////////////////////////////////////////
// TJvBreatheButtons
///////////////////////////////////////////////////////////

{************************************************************}
constructor TJvBreatheButtons.Create(AOwner: TComponent);
begin
   FPlaylist:=TJvBreatheButton.Create(AOwner);
   FPrev:=TJvBreatheButton.Create(AOwner);
   FMove:=TJvBreatheButton.Create(AOwner);
   FPlay:=TJvBreatheButton.Create(AOwner);
   FNext:=TJvBreatheButton.Create(AOwner);
   FExit:=TJvBreatheButton.Create(AOwner);
   FOptions:=TJvBreatheButton.Create(AOwner);
   FOpen:=TJvBreatheButton.Create(AOwner);
   FPause:=TJvBreatheButton.Create(AOwner);
   FId3:=TJvBreatheButton.Create(AOwner);
   FForward:=TJvBreatheButton.Create(AOwner);
   FRewind:=TJvBreatheButton.Create(AOwner);
   FStop:=TJvBreatheButton.Create(AOwner);
   FMinimize:=TJvBreatheButton.Create(AOwner);
end;
{************************************************************}
destructor TJvBreatheButtons.Destroy;
begin
   FPlaylist.Free;
   FPrev.Free;
   FMove.Free;
   FPlay.Free;
   FNext.Free;
   FExit.Free;
   FOptions.Free;
   FOpen.Free;
   FPause.Free;
   FId3.Free;
   FForward.Free;
   FRewind.Free;
   FStop.Free;
   FMinimize.Free;
   inherited;
end;
{************************************************************}

///////////////////////////////////////////////////////////
// TJvBreatheButtons
///////////////////////////////////////////////////////////

{************************************************************}
constructor TJvBreatheButton.Create(AOwner: TComponent);
begin
   FButton:=TJvImage.Create(AOwner);
   FButton.parent:=TWinControl(AOwner);
end;
{************************************************************}
function TJvBreatheButton.GetEnabled: boolean;
begin
   result:=FButton.Enabled;
end;
{************************************************************}
function TJvBreatheButton.GetHint: string;
begin
   result:=FButton.Hint;
end;
{************************************************************}
function TJvBreatheButton.GetMouseDown: TMouseEvent;
begin
   result:=FButton.OnMouseDown;
end;
{************************************************************}
function TJvBreatheButton.GetMouseMove: TMouseMoveEvent;
begin
   result:=FButton.OnMouseMove;
end;
{************************************************************}
function TJvBreatheButton.GetMouseUp: TMouseEvent;
begin
   result:=FButton.OnMouseUp;
end;
{************************************************************}
function TJvBreatheButton.GetOnClick: TNotifyEvent;
begin
   result:=FButton.OnClick;
end;
{************************************************************}
function TJvBreatheButton.GetOnDblClick: TNotifyEvent;
begin
   result:=FButton.OnDblClick;
end;
{************************************************************}
function TJvBreatheButton.GetPopupMenu: TPopupMenu;
begin
   result:=FButton.PopupMenu;
end;
{************************************************************}
function TJvBreatheButton.GetShowHint: boolean;
begin
   result:=FButton.ShowHint;
end;
{************************************************************}
function TJvBreatheButton.GetVisible: boolean;
begin
   result:=FButton.Visible;
end;
{************************************************************}
procedure TJvBreatheButton.SetEnabled(const Value: boolean);
begin
   FButton.Enabled:=Value;
end;
{************************************************************}
procedure TJvBreatheButton.SetHint(const Value: string);
begin
   FButton.Hint:=Value;
end;
{************************************************************}
procedure TJvBreatheButton.SetMouseDown(const Value: TMouseEvent);
begin
   FButton.OnMouseDown:=Value;
end;
{************************************************************}
procedure TJvBreatheButton.SetMouseUp(const Value: TMouseEvent);
begin
   FButton.OnMouseUp:=Value;
end;
{************************************************************}
procedure TJvBreatheButton.SetMouveMove(const Value: TMouseMoveEvent);
begin
   FButton.OnMouseMove:=Value;
end;
{************************************************************}
procedure TJvBreatheButton.SetOnClick(const Value: TNotifyEvent);
begin
   FButton.OnClick:=Value;
end;
{************************************************************}
procedure TJvBreatheButton.SetOnDblClick(const Value: TNotifyEvent);
begin
   FButton.OnDblClick:=Value;
end;
{************************************************************}
procedure TJvBreatheButton.SetPopupMenu(const Value: TPopupMenu);
begin
   FButton.PopupMenu:=Value;
end;
{************************************************************}
procedure TJvBreatheButton.SetShowHint(const Value: boolean);
begin
   FButton.ShowHint:=Value;
end;
{************************************************************}
procedure TJvBreatheButton.SetVisible(const Value: boolean);
begin
   FButton.Visible:=Value;
end;
{************************************************************}

///////////////////////////////////////////////////////////
// TJvBreatheButtons
///////////////////////////////////////////////////////////

{************************************************************}
constructor TJvBreatheScrollLabel.Create(AOwner: TComponent);
begin
   FLabel:=TJvScrollingLabel.Create(AOwner);
   FLabel.Parent:=TwinControl(AOwner);
   FLabel.AutoSize:=false;
   FLabel.ParentColor:=false;
   FLabel.ParentFont:=false;
   FLabel.ParentShowHint:=false;
end;
{************************************************************}
function TJvBreatheScrollLabel.GeTColor: TColor;
begin
   result:=FLabel.color;
end;
{************************************************************}
function TJvBreatheScrollLabel.GetDirection: TLabelDirection;
begin
   result:=FLabel.ScrollDirection;
end;
{************************************************************}
function TJvBreatheScrollLabel.GetFont: TFont;
begin
   result:=FLabel.Font;
end;
{************************************************************}
function TJvBreatheScrollLabel.GetHint: string;
begin
   result:=FLabel.Hint;
end;
{************************************************************}
function TJvBreatheScrollLabel.GetInterval: cardinal;
begin
   result:=FLabel.ScrollInterval;
end;
{************************************************************}
function TJvBreatheScrollLabel.GetNoGrap: boolean;
begin
   result:=FLabel.NoGrap;
end;
{************************************************************}
function TJvBreatheScrollLabel.GetScrolling: boolean;
begin
   result:=FLabel.Scrolling;
end;
{************************************************************}
function TJvBreatheScrollLabel.GetShowHint: boolean;
begin
   result:=FLabel.ShowHint;
end;
{************************************************************}
function TJvBreatheScrollLabel.GetTransparent: boolean;
begin
   result:=FLabel.Transparent;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetCaption(const Value: string);
begin
   FCaption := Value;
   FLabel.Text:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SeTColor(const Value: TColor);
begin
   FLabel.Color:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetDirection(const Value: TLabelDirection);
begin
   FLabel.ScrollDirection:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetFont(const Value: TFont);
begin
   FLabel.Font:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetHeight(const Value: integer);
begin
  FHeight := Value;
  FLabel.Height:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetHint(const Value: string);
begin
   FLabel.Hint:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetInterval(const Value: cardinal);
begin
   FLabel.ScrollInterval:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetLeft(const Value: integer);
begin
  FLeft := Value;
  FLabel.Left:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetNoGrap(const Value: boolean);
begin
   FLabel.NoGrap:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetScrolling(const Value: boolean);
begin
   FLabel.Scrolling:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetShowHint(const Value: boolean);
begin
   FLabel.ShowHint:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetTop(const Value: integer);
begin
   FTop := Value;
   FLabel.Top:=FTop;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetTransparent(const Value: boolean);
begin
   FLabel.Transparent:=Value;
end;
{************************************************************}
procedure TJvBreatheScrollLabel.SetWidth(const Value: integer);
begin
   FWidth := Value;
   FLabel.Width:=Value;
end;
{************************************************************}

end.
