{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvCabFile.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvCabFile;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvTypes, JvHSetup;

type
  TCabInfo = record
    CabinetPath:string;
    CabinetFile:string;
    DiskName:string;
    Id:Shortint;
    CabinetNumber:Shortint;
  end;
  {$EXTERNALSYM TCabInfo}
  TOnCabInfo = procedure (Sender: TObject;CabInfo:TCabInfo) of object;
  {$EXTERNALSYM TOnCabInfo}
  TOnExtracted = procedure (Sender: TObject;Successed:boolean;var Continue:boolean;Source,Dest:string) of object;
  {$EXTERNALSYM TOnExtracted}
  TOnExtractFile = procedure (Sender: TObject;FileName:string;DestPath:string) of object;
  {$EXTERNALSYM TOnExtractFile}
  TOnNeedNewCabinet = procedure (Sender: TObject;var Continue:boolean;CabInfo:TCabInfo;var NewPath:string) of object;
  {$EXTERNALSYM TOnNeedNewCabinet}

  TJvCabFile = class(TComponent)
  private
    FFileName: TFileName;
    FFiles: TstringList;
    FOnCabInfo: TOnCabInfo;
    FOnFiles: TNotifyEvent;
    FOnExtracted: TOnExtracted;
    FDestPath:string;
    FOnExtractFile: TOnExtractFile;
    FOnNeed: TOnNeedNewCabinet;
    FTmpstring : string;
    Fdll:THandle;
    FSetupIterateCabinet:TSetupIterateCabinet;
    procedure SetFileName(const Value: TFileName);
    procedure SetFiles(const Value: TstringList);
    procedure RefreshFiles;
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property FileName:TFileName read FFileName write SetFileName;
    property Files:TstringList read FFiles write SetFiles;
    property OnCabInfo:TOnCabInfo read FOnCabInfo write FOnCabInfo;
    property OnFilesListed:TNotifyEvent read FOnFiles write FonFiles;
    property OnFileExtracted:TOnExtracted read FOnExtracted write FOnExtracted;
    property OnStartFileExtraction:TOnExtractFile read FOnExtractFile write FonExtractFile;
    property OnNeedNewCabinet:TOnNeedNewCabinet read FonNeed write FOnNeed;
    function ExtractAll(destpath:string):boolean;
    function ExtractFile(FileName:string;DestPath:string):boolean;
  end;

implementation

resourcestring
     RC_SetupApiDll      =        'Unable to find setupapi.dll';

{**************************************************}
constructor TJvCabFile.Create(AOwner: TComponent);
begin
   inherited;
   FFiles:=TstringList.Create;
   FFileName:='';
   FDll:=LoadLibrary('setupapi.dll');
   if Fdll<>0 then
      FSetupIterateCabinet:=GetProcAddress(Fdll,'SetupIterateCabinetA')
   else
     raise Exception.Create(RC_SetupApiDll);
end;
{**************************************************}
destructor TJvCabFile.Destroy;
begin
   Ffiles.free;
   if Fdll<>0 then
      FreeLibrary(Fdll);
   inherited;
end;
{**************************************************}
procedure TJvCabFile.SetFileName(const Value: TFileName);
begin
   FFileName := Value;
   RefreshFiles;
end;
{**************************************************}
procedure TJvCabFile.SetFiles(const Value: TstringList);
begin
   //do nothing !!!!
end;
{**************************************************}
function CBack(Context:Pointer;Notification,Param1:Integer;var Param2:integer):Integer;stdcall;
var
   cab:PFILE_IN_CABINET_INFO;
   Sender:TJvCabFile;
   cabinfo:TCabInfo;
begin
   result:=ERROR_BAD_COMMAND;
   if Context<>nil then
      Sender:=TJvCabFile(Context^)
   else
      exit;

   //this callback is only for listing files in a cabinet ... pouet pouet !
   if Notification = SPFILENOTIFY_FILEINCABINET then //found a file in the cabinet
   begin
      result:=FILEOP_SKIP;
      cab:=PFILE_IN_CABINET_INFO(Param1);
      Sender.FFiles.Add(strpas(cab^.NameInCabinet));
   end
   else if Notification = SPFILENOTIFY_CABINETINFO then //give cabinet info
   begin
      if Assigned(Sender.FOnCabInfo)and(Param1<>0) then
      begin
         cabinfo.CabinetPath:=strpas(PCabinet_Info(Param1)^.CabinetPath);
         cabinfo.CabinetFile:=strpas(PCabinet_Info(Param1)^.CabinetFile);
         cabinfo.DiskName:=strpas(PCabinet_Info(Param1)^.DiskName);
         cabinfo.Id:=PCabinet_Info(Param1)^.Id;
         cabinfo.CabinetNumber:=PCabinet_Info(Param1)^.CabinetNumber;
         Sender.FOnCabInfo(Sender,cabinfo);
      end;
      result:=0;
   end;
end;
{**************************************************}
function CExtract(Context:Pointer;Notification,Param1:Integer;var Param2:integer):Integer;stdcall;
var
   cab:PFILE_IN_CABINET_INFO;
   Sender:TJvCabFile;
   cabinfo:TCabInfo;
   continue:boolean;
   FPath:TFilePath;
   Path:string;
   i:Integer;
begin
   result:=ERROR_BAD_COMMAND;
   if Context<>nil then
      Sender:=TJvCabFile(Context^)
   else
      exit;


   //this callback is only for listing files in a cabinet ...
   if notification = SPFILENOTIFY_CABINETINFO then
      result:=0
   else if Notification = SPFILENOTIFY_FILEINCABINET then //found a file in the cabinet
   begin
      try
         result:=FILEOP_DOIT;
         cab:=PFILE_IN_CABINET_INFO(Param1);

         if Sender.FDestPath[length(Sender.FDestPath)]='\' then
         begin
            //extract all
            path:=Sender.FDestPath+strpas(cab^.NameInCabinet);
            for i:=1 to length(path) do
             cab^.FullTargetName[i-1]:=path[i];
            cab^.FullTargetName[length(path)]:=#0;

            if Assigned(Sender.FOnExtractFile) then
               Sender.FOnExtractFile(Sender,cab^.FullTargetName,Sender.FDestPath);
         end
         else
         begin
            //Extract specific file
            if UpperCase(extractFileName(Sender.FDestPath))=UpperCase(strpas(cab^.NameInCabinet)) then
            begin
               path:=Sender.FDestPath;
               for i:=1 to length(path) do
                cab^.FullTargetName[i-1]:=path[i];
               cab^.FullTargetName[length(path)]:=#0;
               if Assigned(Sender.FOnExtractFile) then
                 Sender.FOnExtractFile(Sender,cab^.FullTargetName,Sender.FDestPath);
            end
            else result:=FILEOP_SKIP;
         end;
      except
         result:=FILEOP_SKIP;
      end;
   end
   else if Notification = SPFILENOTIFY_FILEEXTRACTED then
   begin
      Continue:=true;
      if param1<>0 then
         FPath:=PFilePath(param1)^;
      if Assigned(Sender.FOnExtracted) then
         Sender.FOnExtracted(Sender,(FPath.ErrorCode=NO_ERROR),continue,
               StrPas(FPath.Source),strpas(FPath.Target));
      if continue then
         result:=NO_ERROR
      else
         result:=ERROR_BAD_COMMAND;
   end
   else if Notification = SPFILENOTIFY_NEEDNEWCABINET then
   begin
      if param1<>0 then
      begin
         cabinfo.CabinetPath:=strpas(PCabinet_Info(Param1)^.CabinetPath);
         cabinfo.CabinetFile:=strpas(PCabinet_Info(Param1)^.CabinetFile);
         cabinfo.DiskName:=strpas(PCabinet_Info(Param1)^.DiskName);
         cabinfo.Id:=PCabinet_Info(Param1)^.Id;
         cabinfo.CabinetNumber:=PCabinet_Info(Param1)^.CabinetNumber;
         Continue:=true;
         path:='';
         if Assigned(Sender.FonNeed) then
         begin
            Sender.FOnNeed(Sender,Continue,cabinfo,path);
            Sender.FTmpstring:=path;
            param2:=Longint(PChar(Sender.FTmpstring));
         end
         else result:=ERROR_BAD_COMMAND;
      end
      else result:=ERROR_BAD_COMMAND;
   end;
end;
{**************************************************}
procedure TJvCabFile.RefreshFiles;
begin
   FFiles.Clear;
   if @FSetupIterateCabinet<>nil then
     if FSetupIterateCabinet(PChar(FFileName),0,CBack,@self) then
        if Assigned(FonFiles) then
           FOnfiles(self);
end;
{**************************************************}
function TJvCabFile.ExtractAll(destpath: string):boolean;
begin
   if destpath[length(destpath)]<>'\' then
      DestPath:=DestPath+'\';
   FDestPath:=DestPath;
   if @FSetupIterateCabinet<>nil then
     result:=FSetupIterateCabinet(PChar(FFileName),0,CExtract,@self)
   else
     result:=false;
end;
{**************************************************}
function TJvCabFile.ExtractFile(FileName, DestPath: string):boolean;
begin
   if destpath[length(destpath)]<>'\' then
      DestPath:=DestPath+'\';
   FDestPath:=DestPath+FileName;
   if @FSetupIterateCabinet<>nil then
     result:=FSetupIterateCabinet(PChar(FFileName),0,CExtract,@self)
   else
     result:=false;
end;
{**************************************************}
end.
