{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvCaesarCipher.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvCaesarCipher;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, JvTypes;

type
  TJvCaesarCipher = class(TComponent)
  private
    Fn:byte;
    FDecoded : string;
    FEncoded : string;
    procedure SetDecoded(st:string);
    procedure SetEncoded(st:string);
    function Crypte(St:string;n:byte):string;
    function CrypteByte(char, n: byte): byte;
  protected
  public
  published
    property N: byte read Fn write Fn;
    property Encoded: string read Fencoded write SetEncoded;
    property Decoded: string read FDecoded write SetDecoded;
    procedure Decode(It:Tstrings);
    procedure Encode(It:Tstrings);
    function EncodeStream(Value:TStream):TStream;
    function DecodeStream(Value:TStream):TStream;
  end;

implementation

{***********************************************************}
function TJvCaesarCipher.CrypteByte(char:byte;n:byte):byte;
var
  j:Integer;
begin
   j:=char+n;
   if j<0 then
     j:=256-j
   else if j>256 then
     j:=j-256;
   result:=j;
end;
{***********************************************************}
function TJvCaesarCipher.Crypte(St:string;n:byte):string;
var
  i:Integer;
begin
   result:='';
   for i:=1 to length(st) do
      result:=result+char(CrypteByte(ord(st[i]),n));
end;
{***********************************************************}
procedure TJvCaesarCipher.SetDecoded(st:string);
begin
   FDecoded:=st;
   FEncoded:=Crypte(st,Fn);
end;
{***********************************************************}
procedure TJvCaesarCipher.SetEncoded(st:string);
begin
   FEncoded:=st;
   FDecoded:=Crypte(st,-Fn);
end;
{***********************************************************}
procedure TJvCaesarCipher.Decode(It:Tstrings);
var
  j:Integer;
begin
   for j:=1 to it.count-1 do
       it[j]:=Crypte(it[j],-Fn);
end;
{***********************************************************}
procedure TJvCaesarCipher.Encode(it:Tstrings);
var
  j:Integer;
begin
   for j:=1 to it.count-1 do
       it[j]:=Crypte(it[j],Fn);
end;
{***********************************************************}
function TJvCaesarCipher.DecodeStream(Value: TStream): TStream;
var
  buffer:array [0..1024] of byte;
  i,count:Integer;
begin
   result:=TMemoryStream.Create;
   while Value.Position<Value.Size do
   begin
      count:=Value.Read(buffer,1024);
      for i:=0 to count-1 do
        buffer[i]:=CrypteByte(buffer[i],-Fn);
      result.Write(buffer,count);
   end;
end;
{***********************************************************}
function TJvCaesarCipher.EncodeStream(Value: TStream): TStream;
var
  buffer:array [0..1024] of byte;
  i,count:Integer;
begin
   result:=TMemoryStream.Create;
   while Value.Position<Value.Size do
   begin
      count:=Value.Read(buffer,1024);
      for i:=0 to count-1 do
        buffer[i]:=CrypteByte(buffer[i],Fn);
      result.Write(buffer,count);
   end;
end;
{***********************************************************}
end.
