{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvDirectories.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvDirectories;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Registry, 
  JvTypes;

type
  TJvDirectories = class(TComponent)
  private
    FBidon : string;
    buf : array [0..250] of char;
    function GetCurrent:string;
    function GetWd:string;
    function GetSyste:string;
    function GetTem:string;
    function GetValueAtIndex(Index: integer):string;
    function GetProgramFiles: string;
  protected
    function CheckLastChar(Value: string):string;
  public
  published
    property CurrentDirectory:string read GetCurrent write FBidon stored false;
    property WindowsDirectory:string read GetWd write FBidon stored false;
    property SystemDirectory :string read GetSyste write FBidon stored false;
    property TempPath :string read GetTem write Fbidon stored false;
    property ApplicationData:string index 0 read GetValueAtIndex write FBidon stored false;
    property Cache:string index 1 read GetValueAtIndex write FBidon stored false;
    property Cookies:string index 2 read GetValueAtIndex write FBidon stored false;
    property Desktop:string index 3 read GetValueAtIndex write FBidon stored false;
    property Favorites:string index 4 read GetValueAtIndex write FBidon stored false;
    property Fonts:string index 5 read GetValueAtIndex write FBidon stored false;
    property History:string index 6 read GetValueAtIndex write FBidon stored false;
    property NetHood:string index 7 read GetValueAtIndex write FBidon stored false;
    property Personal:string index 8 read GetValueAtIndex write FBidon stored false;
    property Programs:string index 9 read GetValueAtIndex write FBidon stored false;
    property ProgramFiles:string read GetProgramFiles write FBidon stored false;
    property Recent:string index 10 read GetValueAtIndex write FBidon stored false;
    property SendTo:string index 11 read GetValueAtIndex write FBidon stored false;
    property StartMenu:string index 12 read GetValueAtIndex write FBidon stored false;
    property Startup:string index 13 read GetValueAtIndex write FBidon stored false;
    property Templates:string index 14 read GetValueAtIndex write FBidon stored false;
  end;

implementation

resourcestring
   RC_ShellFolders      =   'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders\';

var
 DirectoryList : array [0..14] of string = ('AppData','Cache','Cookies','Desktop',
                 'Favorites','Fonts','History','NetHood','Personal','Programs','Recent',
                 'SendTo','Start Menu','Statup','Templates');

{******************************************************}
function TJvDirectories.GetCurrent:string;
begin
  GetCurrentDirectory(250,buf);
  result:=CheckLastChar(buf);
end;
{******************************************************}
function TJvDirectories.GetWd:string;
begin
  GetWindowsDirectory(buf,250);
  result:=CheckLastChar(buf);
end;
{******************************************************}
function TJvDirectories.GetSyste:string;
begin
  GetSystemDirectory(buf,250);
  result:=CheckLastChar(buf);
end;
{******************************************************}
function TJvDirectories.GetTem:string;
begin
  GetTempPath(250,buf);
  result:=CheckLastChar(buf);
end;
{******************************************************}
function TJvDirectories.GetValueAtIndex(Index: integer): string;
begin
  try
    with TRegistry.Create do
    begin
      OpenKey(RC_ShellFolders,false);
      result:=CheckLastChar(ReadString(DirectoryList[Index]));
      Free;
    end;
  except
    result:='';
  end;
end;
{******************************************************}
function TJvDirectories.CheckLastChar(Value: string): string;
begin
  //Check if the last char is a \, and add one if necessary
  if (length(Value)>0) and (Value[length(Value)]<>'\') then
     Value:=Value+'\';
  result:=Value;
end;
{******************************************************}
function TJvDirectories.GetProgramFiles: string;
begin
  try
    with TRegistry.Create do
    begin
      RootKey:=HKEY_Local_Machine;
      OpenKey('Software\Microsoft\Windows\CurrentVersion\',false);
      result:=CheckLastChar(ReadString('ProgramFilesDir'));
      Free;
    end;
  except
    result:='';
  end;
end;
{******************************************************}
end.
