{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvFontBox.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvFontBox;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvCustomBox,
  Buttons, JvTypes, JvCalculator, Dialogs;

type
  TJvFontBox = class(TJvCustomBox)
  private
    FLastValue: TFont;
    function GetFont: TFont;
    procedure SetFont(Value: TFont);
    function ColorToStr(Value: TColor): string;
    function StrToColor(Value: string): TColor;
  protected
    procedure BtnClick(Sender: TObject);override;
    procedure PreEdit(Sender: TObject);
    procedure PostEdit(Sender: TObject);
    function FontToStr(const Font: TFont):string;
    function StrToFont(const Value: string):TFont;
  public
    constructor Create(AOwner: TComponent);override;
  published
    property Value:TFont read GetFont write SetFont;
  end;


implementation

uses
  TypInfo;

{$R RES_FontBox.res}

{*****************************************************}
constructor TJvFontBox.Create(AOwner: TComponent);
begin
  inherited;
  Button.Glyph.LoadFromResourceName(hInstance,'FONTBOX');
  FLastValue := TFont.Create;
  GetEdit.OnEnter := PreEdit;
  GetEdit.OnExit := PostEdit;
end;
{*****************************************************}
procedure TJvFontBox.SetFont(Value: TFont);
begin
  Edit.Text := FontToStr(Value);
end;
{*****************************************************}
function TJvFontBox.GetFont: TFont;
begin
  try
    result := StrToFont(Edit.Text);
  except
    result := FLastValue;
  end;
end;
{*****************************************************}
procedure TJvFontBox.BtnClick(Sender: TObject);
begin
  with TFontDialog.Create(nil) do
  begin
    Font.Assign(GetFont);
    if Execute then
      SetFont(Font);
    Free;
  end;
end;
{*****************************************************}
procedure TJvFontBox.PostEdit(Sender: TObject);
begin
  try
    FLastValue := StrToFont(Edit.Text);
  except
    SetFont(FLastValue);
    Beep;
  end;
end;
{*****************************************************}
procedure TJvFontBox.PreEdit(Sender: TObject);
begin
  FLastValue.Assign(GetFont);
end;
{*****************************************************}
function TJvFontBox.StrToColor(Value: string):TColor;
begin
  Value := UpperCase(Value);
  if Value='BLACK' then
    result := clBlack
  else if Value='MAROON' then
    result := clMaroon
  else if Value='GREEN' then
    result := clGreen
  else if Value='OLIVE' then
    result := clOlive
  else if Value='NAVY' then
    result := clNavy
  else if Value='PURPLE' then
    result := clPurple
  else if Value='TEAL' then
    result := clTeal
  else if Value='GRAY' then
    result := clGray
  else if Value='SILVER' then
    result := clSilver
  else if Value='RED' then
    result := clRed
  else if Value='LIME' then
    result := clLime
  else if Value='YELLOW' then
    result := clYellow
  else if Value='BLUE' then
    result := clBlue
  else if Value='FUCHSIA' then
    result := clFuchsia
  else if Value='AQUA' then
    result := clAqua
  else if Value='WHITE' then
    result := clWhite
  else
    result := clBlack;
end;
{*****************************************************}
function TJvFontBox.ColorToStr(Value: TColor):string;
begin
  case Value of
    clBlack : result := 'BLACK';
    clMaroon : result := 'MAROON';
    clGreen : result := 'GREEN';
    clOlive : result := 'OLIVE';
    clNavy : result := 'NAVY';
    clPurple : result := 'PURPLE';
    clTeal : result := 'TEAL';
    clGray : result := 'GRAY';
    clSilver : result := 'SILVER';
    clRed : result := 'RED';
    clLime : result := 'LIME';
    clYellow : result := 'YELLOW';
    clBlue : result := 'BLUE';
    clFuchsia : result := 'FUCHSIA';
    clAqua : result := 'AQUA';
    clWhite : result := 'WHITE';
    else
      result := 'BLACK';
  end;
end;
{*****************************************************}
  { FontName
  FontSize
  Color
  Attribs (Bold, Italic, Underline, StrikeOut)}
function TJvFontBox.FontToStr(const Font: TFont): string;
begin
  with TStringList.Create do
  begin
    Add(Font.Name);
    Add(IntToStr(Font.size));
    Add(ColorToStr(Font.Color));
    if fsBold in Font.Style then
      Add('Bold');
    if fsItalic in Font.Style then
      Add('Italic');
    if fsUnderline in Font.Style then
      Add('Underline');
    if fsStrikeOut in Font.Style then
      Add('StrikeOut');
    result := CommaText;
    Free;
  end;
end;
{*****************************************************}
function TJvFontBox.StrToFont(const Value: string): TFont;
var
 i: Integer;
begin
  result := TFont.Create;
  with TStringList.Create do
  begin
    CommaText := Value;
    if Count=0 then
    begin
      Free;
      Exit;
    end
    else if Count<3 then
    begin
      Free;
      raise Exception.Create('');
    end;
    result.Name := Strings[0];
    result.Size := StrToIntDef(Strings[1],result.Size);
    result.Color := StrToColor(Strings[2]);
    for i:=3 to Count-1 do
    begin
      Strings[i] := UpperCase(Strings[i]);
      if Strings[i]='BOLD' then
        result.Style := result.Style+[fsBold]
      else if Strings[i]='ITALIC' then
        result.Style := result.Style+[fsItalic]
      else if Strings[i]='UNDERLINE' then
        result.Style := result.Style+[fsUnderline]
      else if Strings[i]='STRIKEOUT' then
        result.Style := result.Style+[fsStrikeOut]
    end;
    Free;
  end;
end;
{*****************************************************}
end.
