{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvFormAnimatedIcon.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvFormAnimatedIcon;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  ExtCtrls;

type
  TJvFormAnimatedIcon = class(TComponent)
  private
    FForm: TCustomForm;
    FActive: boolean;
    FDelay: cardinal;
    FImgList: TImageList;
    FTimer:TTimer;
    FNumber:Integer;
    procedure SetActive(const Value: boolean);
    procedure SetDelay(const Value: cardinal);
    procedure Animate(Sender: TObject);
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Icons:TImageList read FImgList write FImgList;
    property Active:boolean read FActive write SetActive;
    property Delay:cardinal read FDelay write SetDelay default 100;
  end;

implementation

{**************************************************}
constructor TJvFormAnimatedIcon.Create(AOwner: TComponent);
begin
  inherited;

  FForm := GetParentForm(TControl(AOwner));
  FNumber:=0;
  FDelay:=100;

  if not(csDesigning in ComponentState) then
  begin
    FTimer:=TTimer.Create(self);
    FTimer.OnTimer:=Animate;
    FTimer.Interval:=FDelay;
    FTimer.Enabled:=Factive;
  end;
end;
{**************************************************}
destructor TJvFormAnimatedIcon.Destroy;
begin
  if not(csDesigning in ComponentState) then
    FTimer.Free;
  inherited;
end;
{**************************************************}
procedure TJvFormAnimatedIcon.Animate(Sender: TObject);
begin
  if (FImglist<>nil)and(FImgList.Count<>0) then
  begin
    FNumber := (FNumber+1)mod FimgList.count;
    FimgList.GetIcon(Fnumber,TForm(FForm).Icon);
  end;
end;
{**************************************************}
procedure TJvFormAnimatedIcon.SetActive(const Value: boolean);
begin
  FActive := Value;
  if not(csDesigning in ComponentState) then
    FTimer.Enabled := FActive;
end;
{**************************************************}
procedure TJvFormAnimatedIcon.SetDelay(const Value: cardinal);
begin
  FDelay := Value;
  if not(csDesigning in ComponentState) then
    FTimer.Interval := FDelay;
end;
{**************************************************}
end.
