{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvFormatDrive.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvFormatDrive;

{$ObjExportAll On} 

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  JvBaseDlg, JvTypes, Dialogs;

type
  TJvFormatDrive = class(TJvCommonDialog)
  private
    FOption: TFormatOption;
    FOnCancel: TNotifyEvent;
    FOnError: TNotifyEvent;
    FOnNoFormat: TNotifyEvent;
    FOnSuccess: TNotifyEvent;
    FHandle:THandle;
    FDrive: Char;
    FSHFormatDrive:TSHFormatDrive;
    FDllHandle:THandle;
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Option:TFormatOption read FOption write FOption default shFull;
    property OnError:TNotifyEvent read FOnError write FonError;
    property OnCancel:TNotifyEvent read FOnCancel write FOnCancel;
    property OnNoformat:TNotifyEvent read FOnNoFormat write FOnNoFormat;
    property OnSuccess:TNotifyEvent read FOnSuccess write FOnSuccess;
    property Drive:Char read FDrive write FDrive default 'A';
    function Execute:boolean;override;
    function FormatDrive(Drive:char):boolean;
  end;

implementation

{*********************************************************}
constructor TJvFormatDrive.Create(AOwner: TComponent);
begin
   inherited;
   FOption:=shFull;
   if AOwner is TForm then
      FHandle:=(AOwner as TForm).Handle
   else
      FHandle:=Application.handle;
   FDrive:='A';
   if not(csDesigning in ComponentState) then
   begin
     FDllHandle:=LoadLibrary('SHELL32.DLL');
     if FDllHandle<>0 then
       FSHFormatDrive:=GetProcAddress(FDllHandle,'SHFormatDrive')
     else
       FSHFormatDrive:=nil;
   end
   else
     FDllHandle:=0;
end;
{*********************************************************}
destructor TJvFormatDrive.Destroy;
begin
   if FDllHandle<>0 then
     FreeLibrary(FDllHandle);
   inherited;
end;
{*********************************************************}
function TJvFormatDrive.Execute:boolean;
begin
   if not(csDesigning in ComponentState) then
     result:=FormatDrive(FDrive)
   else
   begin
     ShowMessage('Unavailable in Design Mode');
     result:=false;
   end;
end;
{*********************************************************}
function TJvFormatDrive.FormatDrive(Drive: char):boolean;
var
   d:word;
   o:word;
begin
   result:=false;  
   drive:=LowerCase(Drive)[1];
   if ord(Drive)<=ord('z') then
   begin
      d:=ord(Drive)-ord('a');
      case FOption of
         shQuickFormat :   o:=SHFMT_OPT_QUICKFORMAT;
         shFull :          o:=SHFMT_OPT_FULL;
         shSystemFilesOnly:o:=SHFMT_OPT_SYSONLY;
         else              o:=SHFMT_OPT_FULL;
      end;
      case FSHFormatDrive(FHandle,d,SHFMT_ID_DEFAULT,o) of
         SHFMT_ERROR :    if Assigned(FonError) then FOnError(self);
         SHFMT_CANCEL :   if Assigned(FonCancel) then FOnCancel(self);
         SHFMT_NOFORMAT : if Assigned(FonNoformat) then FOnNoFormat(self);
         else
         begin
            result:=true;
            if Assigned(FOnSuccess) then FOnSuccess(self);
         end;
      end;
   end;
end;
{*********************************************************}
end.
