{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvFunctions.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvFunctions;

interface

uses
  Windows, Graphics, Messages, SysUtils, ShellApi;

//Transform an icon to a bitmap
function IconToBitmap(ico:HIcon):TBitmap;
{$EXTERNALSYM IconToBitmap}

//Open an object with the shell (url or something like that)
procedure OpenObject(Value : PChar);overload;
{$EXTERNALSYM OpenObject}
procedure OpenObject(Value : string);overload;
{$EXTERNALSYM OpenObject}

//Raise the last Exception
procedure RaiseLastWin32;overload;
{$EXTERNALSYM RaiseLastWin32}
procedure RaiseLastWin32(Text: string);overload;
{$EXTERNALSYM RaiseLastWin32}
//Raise the last Exception with a small comment from your part
//Same as linux function ;)
procedure PError(Text: string);
{$EXTERNALSYM PError}

//Return the maximum of three integers
function GetMax(i,j,k:integer):Integer;
{$EXTERNALSYM GetMax}

//Return the minimum of three integers
function GetMin(i,j,k:integer):Integer;
{$EXTERNALSYM GetMin}

//Convert RGB Values to HSV
procedure RgbToHSV(r,g,b:Integer;var h,s,v:integer);
{$EXTERNALSYM RgbToHSV}

//Get version of Shell.dll
function GetShellVersion: Integer;
{$EXTERNALSYM GetShellVersion}


implementation

resourcestring
  SWin32Error = 'Win32 Error.  Code: %d.'#10'%s';

var
  ShellVersion: Integer;

{*****************************************************}
function IconToBitmap(ico:HIcon):TBitmap;
var
 i:TPicture;
begin
  i:=TPicture.Create;
  i.Icon.Handle:=ico;
  result:=TBitmap.Create;
  result.Height:=32;
  result.width:=32;
  result.canvas.draw(0,0,i.icon);
  i.free;
end;
{*****************************************************}
procedure OpenObject(Value : string);
begin
  ShellExecute(0, Nil, PChar(Value), Nil, Nil, SW_NORMAL);
end;
{*****************************************************}
procedure OpenObject(Value : PChar);
begin
  ShellExecute(0, Nil, Value, Nil, Nil, SW_NORMAL);
end;
{**************************************************}
procedure RaiseLastWin32;
begin
  PError('');
end;
{**************************************************}
procedure RaiseLastWin32(Text: string);
begin
  PError(Text);
end;
{**************************************************}
procedure PError(Text: string);
var
 lastError:Integer;
 st:string;
begin
  lastError:=GetLastError;
  if lastError<>0 then
  begin
    st:=Format(SWin32Error,[LastError,SysErrorMessage(LastError)]);
    if (Text<>'') then
      st:=Text+':'+st;
    Raise EWin32Error.Create(st);
  end;
end;
{**************************************************}
function GetMax(i,j,k:integer):Integer;
begin
   if j>i then i:=j;
   if k>i then i:=k;
   result:=i;
end;
{**************************************************}
function GetMin(i,j,k:integer):Integer;
begin
   if j<i then i:=j;
   if k<i then i:=k;
   result:=i;
end;
{**************************************************}
procedure RgbToHSV(r,g,b:Integer;var h,s,v:integer);
var
   Delta:Integer;
   Min,Max:Integer;
begin
   Min:=GetMin(r,g,b);
   Max:=GetMax(r,g,b);
   v:=max;
   Delta := Max-Min;
   if Max=0 then s:=0 else s:=(255*Delta) div Max;
   if s=0 then h:=0 else
   begin
      if r=max then h:=(60*(g-b)) div delta
      else if g=max then h:=120+(60*(b-r)) div delta
      else h:=240+(60*(r-g))div delta;
      if h<0 then h:=h+360;
   end;
end;
{**************************************************}
function GetShellVersion: Integer;
var
  InfoSize, Wnd: DWORD;
  VerBuf: Pointer;
  FI: PVSFixedFileInfo;
  VerSize: DWORD;
begin
  if ShellVersion = 0 then
  begin
    InfoSize := GetFileVersionInfoSize('shell32.dll', Wnd);
    if InfoSize <> 0 then
    begin
      GetMem(VerBuf, InfoSize);
      try
        if GetFileVersionInfo('shell32.dll', Wnd, InfoSize, VerBuf) then
          if VerQueryValue(VerBuf, '\', Pointer(FI), VerSize) then
            ShellVersion := FI.dwFileVersionMS;
      finally
        FreeMem(VerBuf);
      end;
    end;
  end;
  Result := ShellVersion;
end;
{**************************************************}
end.
