{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvGradientCaption.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvGradientCaption;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, JvGradient, JvTypes;

type
  TJvGradientCaption = class(TWinControl)
  private
     FJvGradient1 : TJvGradient;
     FLabel1 : TLabel;
     FHint: boolean;
     function GetJvGradient1Cursor:TCursor;
     procedure SetJvGradient1Cursor(Value:TCursor);
     function GetJvGradient1Hint:string;
     procedure SetJvGradient1Hint(Value:string);
     function GetJvGradient1StarTColor:TColor;
     procedure SetJvGradient1StarTColor(Value:TColor);
     function GetJvGradient1EndColor:TColor;
     procedure SetJvGradient1EndColor(Value:TColor);
     function GetJvGradient1Steps:Integer;
     procedure SetJvGradient1Steps(Value:integer);
     function GetLabel1Left:Integer;
     procedure SetLabel1Left(Value:integer);
     function GetLabel1Top:Integer;
     procedure SetLabel1Top(Value:integer);
     function GetLabel1Cursor:TCursor;
     procedure SetLabel1Cursor(Value:TCursor);
     function GetLabel1Hint:string;
     procedure SetLabel1Hint(Value:string);
     function GetLabel1Caption:string;
     procedure SetLabel1Caption(Value:string);
     function GetLabel1Color:TColor;
     procedure SetLabel1Color(Value:TColor);
     procedure SetHints(const Value: boolean);
     function GetFont: Tfont;
     procedure SetFont(const Value: Tfont);
    function GetGstyle: TGradstyle;
    procedure SetGstyle(const Value: TGradstyle);
    function GetAlignment: TAlignment;
    procedure Setalignment(const Value: TAlignment);
  protected
  public
     constructor Create(AOwner: TComponent);override;
     destructor Destroy;override;
  published
     property GradientCursor:TCursor read GetJvGradient1Cursor write SetJvGradient1Cursor;
     property GradientHint:string read GetJvGradient1Hint write SetJvGradient1Hint;
     property GradientStarTColor:TColor read GetJvGradient1StarTColor write SetJvGradient1StarTColor default clBlack;
     property GradientEndColor:TColor read GetJvGradient1EndColor write SetJvGradient1EndColor default clWhite;
     property GradientSteps:Integer read GetJvGradient1Steps write SetJvGradient1Steps default 100;
     property LabelLeft:Integer read GetLabel1Left write SetLabel1Left default 18;
     property LabelTop:Integer read GetLabel1Top write SetLabel1Top default 8;
     property LabelCursor:TCursor read GetLabel1Cursor write SetLabel1Cursor;
     property LabelHint:string read GetLabel1Hint write SetLabel1Hint;
     property LabelCaption:string read GetLabel1Caption write SetLabel1Caption;
     property LabelColor:TColor read GetLabel1Color write SetLabel1Color;
     property LabelFont:Tfont read GetFont write SetFont;
     property ShowHint:boolean read FHint write SetHints default false;
     property Gradientstyle:TGradstyle read GetGstyle write SetGstyle;
     property LabelAlignment:TAlignment read GetAlignment write Setalignment;
     property Align;
  end;

implementation

resourcestring
   RC_YourTextHere     =     'Put your text here ...';

{***************************************************}
constructor TJvGradientCaption.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   self.width:=285;
   self.height:=30;
   FJvGradient1:=TJvGradient.Create(self);
   FJvGradient1.parent:=self;
   FLabel1:=TLabel.Create(self);
   FLabel1.parent:=self;
   FJvGradient1.Left:=0;
   FJvGradient1.Top:=0;
   FJvGradient1.StarTColor:=clBlack;
   FJvGradient1.EndColor:=clWhite;
   FJvGradient1.Steps:=100;
   FLabel1.Left:=10;
   FLabel1.Top:=8;
   FLabel1.Color:=clBlue;
   FLabel1.Transparent:=true;
   FLabel1.font.color:=clwhite;
   FLabel1.caption:=RC_YourTextHere;
   FHint:=false;
end;
{***************************************************}
destructor TJvGradientCaption.Destroy;
begin
   FJvGradient1.free;
   FLabel1.free;

   inherited Destroy;
end;
{***************************************************}
function TJvGradientCaption.GetJvGradient1Cursor:TCursor;
begin
   result:=FJvGradient1.Cursor;
end;
{***************************************************}
procedure TJvGradientCaption.SetJvGradient1Cursor(Value:TCursor);
begin
   FJvGradient1.Cursor:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetJvGradient1Hint:string;
begin
   result:=FJvGradient1.Hint;
end;
{***************************************************}
procedure TJvGradientCaption.SetJvGradient1Hint(Value:string);
begin
   FJvGradient1.Hint:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetJvGradient1StarTColor:TColor;
begin
   result:=FJvGradient1.StarTColor;
end;
{***************************************************}
procedure TJvGradientCaption.SetJvGradient1StarTColor(Value:TColor);
begin
   FJvGradient1.StarTColor:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetJvGradient1EndColor:TColor;
begin
   result:=FJvGradient1.EndColor;
end;
{***************************************************}
procedure TJvGradientCaption.SetJvGradient1EndColor(Value:TColor);
begin
   FJvGradient1.EndColor:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetJvGradient1Steps:Integer;
begin
   result:=FJvGradient1.Steps;
end;
{***************************************************}
procedure TJvGradientCaption.SetJvGradient1Steps(Value:integer);
var
   x:TColor;
begin
   FJvGradient1.Steps:=Value;
   x:=GetLabel1Color;
   SetLabel1color(clLime);
   SetLabel1color(x);
end;
{***************************************************}
function TJvGradientCaption.GetLabel1Left:Integer;
begin
   result:=FLabel1.Left;
end;
{***************************************************}
procedure TJvGradientCaption.SetLabel1Left(Value:integer);
begin
   FLabel1.Left:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetLabel1Top:Integer;
begin
   result:=FLabel1.Top;
end;
{***************************************************}
procedure TJvGradientCaption.SetLabel1Top(Value:integer);
begin
   FLabel1.Top:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetLabel1Cursor:TCursor;
begin
   result:=FLabel1.Cursor;
end;
{***************************************************}
procedure TJvGradientCaption.SetLabel1Cursor(Value:TCursor);
begin
   FLabel1.Cursor:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetLabel1Hint:string;
begin
   result:=FLabel1.Hint;
end;
{***************************************************}
procedure TJvGradientCaption.SetLabel1Hint(Value:string);
begin
   FLabel1.Hint:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetLabel1Caption:string;
begin
   result:=FLabel1.Caption;
end;
{***************************************************}
procedure TJvGradientCaption.SetLabel1Caption(Value:string);
begin
   FLabel1.Caption:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetLabel1Color:TColor;
begin
   result:=FLabel1.Color;
end;
{***************************************************}
procedure TJvGradientCaption.SetLabel1Color(Value:TColor);
begin
   FLabel1.Color:=Value;
end;
{***************************************************}
procedure TJvGradientCaption.SetHints(const Value: boolean);
begin
   FHint := Value;
   FLabel1.ShowHint:=Value;
   FJvGradient1.showhint:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetFont: Tfont;
begin
   result:=FLabel1.Font;
end;
{***************************************************}
procedure TJvGradientCaption.SetFont(const Value: Tfont);
begin
   Flabel1.font:=Value;
end;
{***************************************************}
function TJvGradientCaption.GetGstyle: TGradstyle;
begin
   result:=FJvGradient1.style;
end;
{***************************************************}
procedure TJvGradientCaption.SetGstyle(const Value: TGradstyle);
var
   x:TColor;
begin
   FJvGradient1.style:=Value;
   x:=GetLabel1Color;
   SetLabel1color(clLime);
   SetLabel1color(x);
end;
{***************************************************}
function TJvGradientCaption.GetAlignment: TAlignment;
begin
   result:=FLabel1.Alignment;
end;
{***************************************************}
procedure TJvGradientCaption.Setalignment(const Value: TAlignment);
begin
   FLabel1.Alignment:=Value;
end;
{***************************************************}

end.
