{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvKeyboardStates.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvKeyboardStates;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  JvTypes, ExtCtrls;

type
  TJvKeyboardStates = class(TComponent)
  private
    FTimer:TTimer;
    FAnimation:TJvAnimations;
    F1,F2,F3:boolean;
    FCtrl:boolean;
    procedure SetNumLock(Value:boolean);
    function GetNumLock:boolean;
    procedure SetScroll(Value:boolean);
    function GetScroll:boolean;
    procedure SetCapsLock(Value:boolean);
    function GetCapsLock:boolean;
    procedure OnAnimation(Sender: TObject);
    function GetState(Value:integer):boolean;
    procedure SetState(key:Integer;Value:boolean);
    function GetEnabled:boolean;
    procedure SetEnabled(Value:boolean);
    function GetInterval:Cardinal;
    procedure SetInterval(Value:Cardinal);
    procedure SetCtrl(Value:boolean);
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Enabled:boolean read GetEnabled write SetEnabled default false;
    property Interval:Cardinal read GetInterval write SetInterval default 100;
    property Animation:TJvAnimations read FAnimation write FAnimation;
    property NumLock:boolean Read GetNumLock write SetNumLock;
    property ScrollLock:boolean read GetScroll write SetScroll;
    property CapsLock:boolean read GetCapsLock write SetCapsLock;
    property SystemKeys_Enabled:boolean read FCtrl write SetCtrl;
  end;

implementation

{*****************************************************}
procedure TJvKeyboardStates.OnAnimation(Sender: TObject);
begin
   case FAnimation of
        anLeftRight: begin
                        FTimer.Tag:=(Ftimer.Tag+1)mod 4;
                        SetNumLock(Ftimer.Tag=1);
                        SetCapsLock(Ftimer.tag=2);
                        SetScroll(Ftimer.tag=3);
                     end;
        anRightLeft: begin
                        FTimer.Tag:=(Ftimer.tag+1)mod 4;
                        SetNumLock(Ftimer.tag=3);
                        SetCapsLock(Ftimer.tag=2);
                        SetScroll(Ftimer.Tag=1);
                     end;
        anRightAndLeft: begin
                           FTimer.tag:=(FTimer.tag+1)mod 8;
                           SetNumLock((Ftimer.tag=1)or(Ftimer.tag=7));
                           SetCapsLock((Ftimer.tag=2)or(Ftimer.tag=6));
                           SetScroll((Ftimer.tag=3)or(Ftimer.tag=5));
                        end;
        anLeftVumeter:  begin
                           FTimer.tag:=(FTimer.tag+1)mod 4;
                           SetNumLock(FTimer.tag>0);
                           SetCapsLock(FTimer.tag>1);
                           SetScroll(FTimer.tag>2);
                        end;
        anRightVumeter: begin
                           FTimer.tag:=(FTimer.tag+1)mod 4;
                           SetNumLock(FTimer.tag>2);
                           SetCapsLock(FTimer.tag>1);
                           SetScroll(FTimer.tag>0);
                        end;
   end;
end;
{*****************************************************}
constructor TJvKeyboardStates.Create(AOwner: TComponent);
begin
   FTimer:=TTimer.Create(self);
   FTimer.Enabled:=false;
   FTimer.Interval:=100;
   Ftimer.OnTimer:=OnAnimation;
   F1:=GetNumLock;
   F2:=GetCapsLock;
   F3:=GetScroll;
   FCtrl:=true;
   inherited;
end;
{*****************************************************}
procedure TJvKeyboardStates.SetCtrl(Value:boolean);
var
  dummy : LongInt;
begin
   FCtrl:=Value;
   if Value then
     SystemParametersInfo( SPI_SCREENSAVERRUNNING, 0, @Dummy, 0)
   else
     SystemParametersInfo( SPI_SCREENSAVERRUNNING, 1, @Dummy, 0);
end;
{*****************************************************}
destructor TJvKeyboardStates.Destroy;
begin
   if Ftimer.enabled then SetEnabled(false);
   FTimer.free;
   inherited;
end;
{*****************************************************}
function TJvKeyboardStates.GetEnabled:boolean;
begin
   result:=FTimer.Enabled;
end;
{*****************************************************}
procedure TJvKeyboardStates.SetEnabled(Value:boolean);
begin
   if Value then
   begin
      if not(FTimer.Enabled) then
      begin
         F1:=GetNumLock;
         F2:=GetCapsLock;
         F3:=GetScroll;
      end;
      FTimer.Enabled:=true;
      FTimer.Tag:=0;
   end
   else
   begin
      FTimer.Enabled:=false;
      SetNumLock(F1);
      SetCapsLock(F2);
      SetScroll(F3);
   end;
end;
{*****************************************************}
function TJvKeyboardStates.GetInterval:Cardinal;
begin
   result:=FTimer.Interval;
end;
{*****************************************************}
procedure TJvKeyboardStates.SetInterval(Value:Cardinal);
begin
   Ftimer.Interval:=Value;
end;
{*****************************************************}
function TJvKeyboardStates.GetState(Value:integer):boolean;
Var
  KeyState  :  TKeyboardState;
begin
   GetKeyboardState(KeyState);
   result:=KeyState[Value] = 1;
end;
{*****************************************************}
procedure TJvKeyboardStates.SetState(key:Integer;Value:boolean);
Var
  KeyState  :  TKeyboardState;
begin
   GetKeyboardState(KeyState);
   KeyState[key] := integer(Value);
   SetKeyboardState(KeyState);
end;
{*****************************************************}
procedure TJvKeyboardStates.SetNumLock(Value:boolean);
begin
   SetState(VK_NUMLOCK,Value);
end;
{*****************************************************}
function TJvKeyboardStates.GetNumLock:boolean;
begin
   result:=GetState(VK_NUMLOCK);
end;
{*****************************************************}
procedure TJvKeyboardStates.SetScroll(Value:boolean);
begin
   SetState(VK_SCROLL,Value);
end;
{*****************************************************}
function TJvKeyboardStates.GetScroll:boolean;
begin
   result:=GetState(VK_SCROLL);
end;
{*****************************************************}
procedure TJvKeyboardStates.SetCapsLock(Value:boolean);
begin
   SetState(20,Value);
end;
{*****************************************************}
function TJvKeyboardStates.GetCapsLock:boolean;
begin
   result:=GetState(20);
end;
{*****************************************************}
end.
