{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvListView.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvListView;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  ComCtrls, CommCtrl, Menus;

type
  TJvSortMethod = (smAutomatic, smAlphabetic, smNonCaseSensitive, smNumeric, smDate, smTime, smDateTime, smCurrency);
  TOnSortMethod = function (Sender: TObject; Column: integer):TJvSortMethod of object;
  TProgress = procedure (Sender: TObject; Progression,Total: integer) of object;

  TJvListItem = class (TListItem)
  private
    FPopupMenu: TPopupMenu;
    FBold: boolean;
  protected
    procedure SetPopupMenu(const Value: TPopupMenu);
  public
    constructor CreateEnh(AOwner: TListItems);
    property PopupMenu:TPopupMenu read FPopupMenu write SetPopupMenu;
  end;
  {$EXTERNALSYM TJvListItem}

  TJvListView = class(TListView)
  private
    FColor,FSaved:TColor;
    FOnMouseEnter : TNotifyEvent;
    FOnMouseLeave : TNotifyEvent;
    FOnParentColorChanged : TNotifyEvent;
    FOnCtl3DChanged : TNotifyEvent;
    FOver:boolean;
    FSort: boolean;
    FLast:Integer;
    FOnSave: TProgress;
    FOnLoad: TProgress;
    FOnSort: TOnSortMethod;
    FOnHScroll: TNotifyEvent;
    FOnVScroll: TNotifyEvent;
  protected
    function CreateListItem: TListItem; override;
    procedure ColClick(Column: TListColumn); override;
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
    procedure MouseEnter(var msg:Tmessage); message CM_MOUSEENTER;
    procedure MouseLeave(var msg:Tmessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorChangeD;
    procedure WMNOTIFY(var msg : TWMNotify);message CN_NOTIFY;
  public
    constructor Create(AOwner: TComponent);override;
  published
    property HintColor:TColor read FColor write FColor default clInfoBk;
    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
    property OnCtl3DChanged : TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property SortOnClick:boolean read FSort write FSort default true;
    procedure SetItemPopup(Node:TListItem;Value:TPopupMenu);
    function GetItemPopup(Node:TListItem):TPopupMenu;
    procedure SaveToFile(FileName:string;ForceOldStyle:boolean=false);
    procedure LoadFromFile(FileName:string);
    procedure SaveToStream(stream:TStream;ForceOldStyle:boolean=false);
    procedure LoadFromStream(stream:TStream);
    procedure SaveToCSV(FileName:string;Separator:char=';');
    procedure LoadFromCSV(FileName:string;Separator:char=';');
    property OnLoadProgress:TProgress read FOnLoad write FOnLoad;
    property OnSaveProgress:TProgress read FOnSave write FOnSave;
    property OnAutoSort:TOnSortMethod read FOnSort write FOnSort;
    {$IFNDEF D6}
    procedure SelectAll;
    {$ENDIF}
    procedure UnselectAll;
    procedure InvertSelection;
    {$IFNDEF D6}
    procedure DeleteSelected;
    {$ENDIF}
    property OnVerticalScroll:TNotifyEvent read FOnVScroll write FOnVScroll;
    property OnHorizontalScroll:TNotifyEvent read FOnHScroll write FOnHScroll;
  end;

implementation

{**************************************************}
constructor TJvListItem.CreateEnh(AOwner: TListItems);
begin
   Create(Aowner);
   FBold:=false;
   FPopupMenu:=TPopupMenu.Create(AOWner.Owner);
end;
{**************************************************}
procedure TJvListItem.SetPopupMenu(const Value: TPopupMenu);
begin
   FPopupMenu:=Value;
end;
{**************************************************}




{***********************************************}
constructor TJvListView.Create(AOwner: TComponent);
begin
   FColor:=clInfoBk;
   FOver:=false;
   FSort:=true;
   FLast:=-1;
   inherited Create(AOwner);
   Controlstyle := Controlstyle + [csAcceptsControls];
end;
{***********************************************}
procedure TJvListView.WMNOTIFY(var msg : TWMNotify);
var
  Node:TListItem;
  Point:TPoint;
begin
   inherited;

   Point:=Mouse.CursorPos;
   Point:=ScreenToClient(point);
   with Msg,point do
   begin
      case NMHdr^.code of
           NM_CLICK,NM_RCLICK : begin
                                   Node:=GetItemAt(x,y);
                                   if Assigned(Node) then
                                      Selected:=Node;
                                   if (Selected<>nil)and(NMHdr^.code=NM_RCLICK) then
                                        TJvListItem(selected).PopupMenu.Popup(Mouse.CursorPos.x,Mouse.CursorPos.Y);
                                end;
      end;
   end;
end;
{**************************************************}
procedure TJvListView.WMHScroll(var Message: TWMHScroll);
begin
   inherited;
   if Assigned(FOnHScroll) then
      FOnHScroll(self);
end;
{**************************************************}
procedure TJvListView.WMVScroll(var Message: TWMVScroll);
begin
   inherited;
   if Assigned(FOnVScroll) then
      FOnVScroll(self);
end;
{***********************************************}
procedure TJvListView.MouseEnter(var msg:Tmessage);
begin
   FOver:=true;
   FSaved:=Application.HintColor;
   Application.HintColor:=FColor;
   if Assigned(FonMouseEnter) then FonMouseEnter(self);
end;
{**************************************************}
procedure TJvListView.MouseLeave(var msg:Tmessage);
begin
   Application.HintColor:=FSaved;
   FOver:=false;
   if Assigned(FonMouseLeave) then FonMouseLeave(self);
end;
{**************************************************}
procedure TJvListView.CMCtl3DChanged(var Message: TMessage);
begin
   inherited;
   if Assigned(FonCtl3DChanged) then FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvListView.CMParentColorChanged(var Message: TMessage);
begin
   inherited;
   if Assigned(FonParentColorChanged) then FonParentColorChanged(Self);
end;
{******************************************************************************}
function FirstNonAlpha(Value:string):Integer;
var
  len:Integer;
  i,j:Integer;
  comma:boolean;
begin
   len:=Length(Value);
   i:=1;j:=0;
   comma:=false;
   
   while (i<=len) do
   begin
     case Value[i] of
       '0','1','2','3','4','5','6','7','8','9' : j:=i;
       ',','.' :   if not comma then
                     comma:=true
                   else
                   begin
                     j:=i-1;
                     i:=Len;
                   end;
       else
           begin
             j:=i-1;
             i:=Len;
           end;
     end;
     inc(i);
   end;
   
   result:=j;
end;
{******************************************************************************}
function IsBigger(First,Second:string;SortType:TJvSortMethod):boolean;
var
  i,j:real;
  d,e:TDateTime;
  a,b:Currency;
  l,m:int64;
  st,st2:string;
  int1,int2:Integer;
begin
   result:=false;
   if Trim(First)='' then result:=false
   else if Trim(Second)='' then result:=true
   else
   begin
      case SortType of
          smAlphabetic       : result:=First>Second;
          smNonCaseSensitive : result:=UpperCase(First)>UpperCase(Second);
          smNumeric :    begin
                            try
                              i:=StrToFloat(First);
                              j:=StrToFloat(Second);
                              result:=i>j;
                            except
                              try
                                l:=StrToInt64(First);
                              except
                                l:=0;
                              end;
                              try
                                m:=StrToInt64(Second);
                              except
                                m:=0;
                              end;
                              result:=l>m;
                            end;
                         end;
          smDate :       begin
                            d:=StrToDate(First);
                            e:=StrToDate(Second);
                            result:=d>e;
                         end;
          smTime :       begin
                            d:=StrToTime(First);
                            e:=StrToTime(Second);
                            result:=d>e;
                         end;
          smDateTime :   begin
                            d:=StrToDateTime(First);
                            e:=StrToDateTime(Second);
                            result:=d>e;
                         end;
          smCurrency :   begin
                            a:=StrToCurr(First);
                            b:=StrToCurr(Second);
                            result:=a>b;
                         end;
          smAutomatic :  begin
                            int1:=FirstNonAlpha(First);
                            int2:=FirstNonAlpha(Second);
                            if (int1<>0) and (int2<>0) then
                            begin
                              st:=Copy(First,1,int1);
                              st2:=Copy(Second,1,int2);
                              try
                                result:=StrToFloat(st)>StrToFloat(st2);
                              except
                                result:=First>Second;
                              end;
                            end
                            else
                              result:=First>Second;
                         end;
      end;
   end;
end;
{******************************************************************************}
function IsSmaller(First,Second:string;SortType:TJvSortMethod):boolean;
begin
   result:=IsBigger(Second,First,SortType);
end;
{******************************************************************************}
function SortList(item1,item2, Paramsort:integer):Integer stdcall;
type
  TParamSort = record
         index:Integer;
         Sender: TObject;
  end;
var
  parm:TParamSort;
  i1,i2: TListItem;
  s1,s2: string;
  i:Integer;
  sortype:TJvSortMethod;
begin
   parm:=TParamSort(Pointer(ParamSort)^);
   i1:=TListItem(item1);
   i2:=TListItem(item2);
   i:=parm.index;

   if Assigned(TJvListView(parm.Sender).FOnSort) then
      sortype:=TJvListView(parm.Sender).FOnSort(parm.Sender,parm.index)
   else
      sortype:=smAutomatic;

   case i of
        {sort by caption}
    0: begin
          s1:=i1.caption;
          s2:=i2.caption;

          if IsBigger(s1,s2,sortype) then result:=+1
          else
             if IsSmaller(s1,s2,sortype) then result:=-1
             else result:=0;
       end;
       {sort by column}
    else
       begin
          if i>i1.SubItems.count then
          begin
             if i>i2.SubItems.count then result:=0
             else result:=-1;
          end
          else if i>i2.SubItems.count then
               result:=+1
          else
          begin
             s1:=i1.subitems[i-1];
             s2:=i2.subitems[i-1];
             if IsBigger(s1,s2,sortype) then result:=+1
             else
               if IsSmaller(s1,s2,sortype) then result:=-1
               else result:=0;
          end;
       end;
    end;
end;
{******************************************************************************}
function SortList2(item1,item2, Paramsort:integer):Integer stdcall;
begin
   result:=-SortList(Item1,item2,ParamSort);
end;
{**************************************************}
procedure TJvListView.ColClick(Column: TListColumn);
type
  TParamSort = record
         index:Integer;
         Sender: TObject;
  end;
var
  parm:TParamSort; 
begin
   inherited;
   if Fsort then
   begin
      parm.index:=column.index;
      parm.Sender:=self;  
      if FLast=Column.index then
      begin
         FLast:=-1;
         customsort(sortlist2,integer(@parm));
      end
      else
      begin
         FLast:=Column.index;
         customsort(sortlist,integer(@parm));
      end;
   end;
end;
{**************************************************}
function TJvListView.CreateListItem: TListItem;
begin
   result:=TJvListItem.CreateEnh(Items);
end;
{**************************************************}
function TJvListView.GetItemPopup(Node: TListItem): TPopupMenu;
begin
   result:=TJvListItem(Node).PopupMenu;
end;
{**************************************************}
procedure TJvListView.SetItemPopup(Node: TListItem; Value: TPopupMenu);
begin
   TJvListItem(Node).PopupMenu:=Value;
end;
{**************************************************}
procedure TJvListView.LoadFromFile(FileName: string);
var
   stream:TFileStream;
begin
   Stream:=TFileStream.Create(FileName,fmOpenRead or fmShareDenyWrite);
   LoadFromStream(stream);
   Stream.free;
end;
{**************************************************}
procedure TJvListView.LoadFromStream(stream: TStream);
var
  buf:array [0..100] of char;
  start:Integer;

  procedure LoadOldStyle(Stream: TStream);
  var
    i,j,k:Integer;
    buf:array [0..100] of byte;
    st:string;
    ch1,checks:boolean;
    t:TListItem;
  begin
    i:=Stream.Position;
    t:=nil;
    st:='';
    self.Items.Clear;
    if Assigned(FOnLoad) then
       FOnLoad(self,0,stream.size-start);
    checks:=false;
    ch1:=CheckBoxes;
    while (i<Stream.Size) do
    begin
       j:=Stream.Read(buf,100);
       if Assigned(FOnLoad) then
          FOnLoad(self,j,stream.size-start);
       i:=i+j;
       k:=0;
       while k<j do
       begin
          while (k<j)and(buf[k]<>0)and(buf[k]<>1) do
          begin
             st:=st+char(buf[k]);
             inc(k);
          end;

          if (k<j) then
          begin
             if t<>nil then
                t.SubItems.Add(st)
             else
             begin
                t:=self.items.add;
                checks:=checks or (st[1]='T');
                if st[1]='T' then t.Checked:=true else t.checked:=false;
                st:=Copy(st,2,length(st));
                t.Caption:=st;
             end;
             if (buf[k]=1) then
                  t:=nil;
             st:='';
          end;
          inc(k);
       end;
    end;
    if (ch1=false) and (checks=false) then
       CheckBoxes:=false;
  end;

  procedure LoadNewStyle(Stream: TStream);
  const
    LV_HASCHECKBOXES = $80;
    LV_CHECKED       = $8000;
  var
    count,i,j:word;
    options:byte;
    st:string;
    t:TListItem;
    buf:array[0..2048] of char;
  begin
    try
     self.Items.BeginUpdate;
     self.Items.Clear;
     self.Items.EndUpdate;

     Stream.Read(options,sizeof(options));
     CheckBoxes := (options and  LV_HASCHECKBOXES)=LV_HASCHECKBOXES;

     //Read all lines
     while Stream.Position<Stream.Size do
     begin
       Stream.Read(count,sizeof(count));

       //statistics
       if Assigned(FOnLoad) then
         FOnLoad(self,Stream.Position,stream.size-start);

       //Read all columns
       t:=Self.Items.Add;
       for i:=1 to count do
       begin
         //Read size of the string
         Stream.Read(j,sizeof(i));

         //Read the string
         ZeroMemory(@buf,sizeof(buf));
         Stream.Read(buf,j);
         st:=buf;

         if (i=1) then
         begin
           t.Caption:=st;
           if CheckBoxes then
             t.Checked := (i and LV_CHECKED)=LV_CHECKED;
         end
         else
           t.SubItems.Add(st);
       end;
     end;
    except
    end;
  end;

begin
   start:=Stream.Position;
   Stream.Read(buf,10);
   buf[10] := #0;
   if (buf<>'LISTVIEW01') then
   begin
     Stream.Position:=start;
     LoadOldStyle(Stream);
   end
   else
     LoadNewStyle(Stream);
end;
{**************************************************}
procedure TJvListView.SaveToFile(FileName: string;ForceOldStyle:boolean);
var
   stream:TFileStream;
begin
   Stream:=TFileStream.Create(FileName,fmCreate or fmShareExclusive);
   SaveToStream(Stream,ForceOldStyle);
   Stream.free;
end;
{**************************************************}
procedure TJvListView.SaveToStream(stream: TStream;ForceOldStyle:boolean);

   procedure SaveOldStyle(Stream: TStream);
   var
     i,j,k:Integer;
     b,c,d,e:byte;
     st:string;
     buf:array[0..1000] of byte;
   begin
     b:=0;
     c:=1;
     d:=ord('T'); //checked
     E:=ord('F'); //not checked
     if Assigned(FOnSave) then
         FOnSave(self,0,self.items.count);
     for i:=0 to self.items.count-1 do
     begin
        if Assigned(FOnSave) then
          FOnSave(self,i+1,self.items.count);
        st:=self.items[i].caption;
        for k:=1 to length(st) do
            buf[k-1]:=byte(st[k]);
        k:=length(st);
        //write checked,not
        if self.Items[i].Checked then Stream.write(d,1) else Stream.write(e,1);
        stream.Write(buf,k);
        if self.items[i].SubItems.Count=0 then
           Stream.write(c,1)
        else
        begin
           Stream.Write(b,1);
           for j:=0 to self.items[i].subitems.count-2 do
           begin
              st:=self.items[i].subitems[j];
              for k:=1 to length(st) do
                  buf[k-1]:=byte(st[k]);
              k:=length(st);
              stream.write(buf,k);
              stream.write(b,1);
           end;
           j:=self.items[i].subitems.count-1;
           st:=self.items[i].subitems[j];
           for k:=1 to length(st) do
               buf[k-1]:=byte(st[k]);
           k:=length(st);
           stream.write(buf,k);
           stream.write(c,1);
        end;
     end;
   end;

   procedure SaveNewStyle(Stream: TStream);
   const
    LV_HASCHECKBOXES = $80;
    LV_CHECKED       = $8000;
   var
    buf:array [0..100] of char;
    i,j:word;
    options:byte;

    procedure WriteString(Txt: String);
    var
     i:word;
     buf:array [1..2056] of char;
    begin
      i:=length(Txt);
      CopyMemory(@buf,@Txt[1],i);
      Stream.Write(i,sizeof(i));
      Stream.Write(buf,i);
    end;

   begin
     buf:='LISTVIEW01';
     Stream.Write(buf,10);
     if CheckBoxes then options := LV_HASCHECKBOXES
     else options := 0;
     Stream.Write(options,sizeof(options));
     for i:=0 to self.Items.Count-1 do
       with Items[i] do
       begin
         j:=SubItems.Count+1;
         Stream.Write(j,sizeof(j));
         WriteString(Caption);
         for j:=0 to SubItems.Count-1 do
           WriteString(SubItems[j]);
       end;
   end;
   
begin
   if ForceOldStyle then
     SaveOldStyle(Stream)
   else
     SaveNewStyle(Stream);
end;
{**************************************************}
procedure TJvListView.LoadFromCSV(FileName: string; Separator: char);
var
   st,st2:string;
   fich:textfile;
   size,current:Integer;
   t:TListItem;
   f:file of byte;
   i,j,k,l:Integer;
begin
   self.Items.Clear;

   AssignFile(f,FileName);
   Reset(f);
   size:=FileSize(f);
   CloseFile(f);

   AssignFile(fich,FileName);
   reset(fich);
   if Assigned(FOnLoad) then
      FOnLoad(self,0,size);
   current:=0;
   while not(eof(fich)) do
   begin
      ReadLn(fich,st);
      current:=current+length(st)+2;
      if Assigned(FOnLoad) then
         FOnLoad(self,current,size);
      t:=self.items.add;

      j:=0;k:=1;
      for i:=1 to length(st) do
         if st[i]='"' then j:=(j+1)mod 2
         else if st[i]=Separator then if j=0 then inc(k);
      if (k<>1) then
      begin
         i:=pos(Separator,st);
         j:=pos('"',st);
         l:=0;

         while i<>0 do
         begin
            if (j=0)or(j>i) then
            begin
               st2:=Copy(st,1,i-1);
               st:=Copy(st,i+1,Length(st));  
            end
            else
            begin
               st:=Copy(st,j+1,Length(st));
               j:=Pos('"',st);
               if j=0 then
               begin
                  st2:=st;
                  st:='';
               end
               else
               begin
                  st2:=Copy(st,1,j-1);
                  st:=Copy(st,j+1,length(st));
                  j:=pos(Separator,st);
                  st:=Copy(st,j+1,length(st));
               end;
            end;
            if l=0 then
            begin
               t.Caption:=st2;
               inc(l);
            end
            else
               t.Subitems.Add(st2);
            dec(k);

            i:=pos(Separator,st);
            j:=pos('"',st);
         end;

         if k=1 then
           t.Subitems.Add(st);
      end
      else
      begin
         if pos('"',st)=0 then
           t.Caption:=st
         else
            st:=Copy(st,pos('"',st)+1,length(st));
            if pos('"',st)=0 then
              t.Caption:=st
            else
              t.Caption:=Copy(st,1,pos('"',st)-1);
      end;
   end;
   CloseFile(fich);
end;
{**************************************************}
procedure TJvListView.SaveToCSV(FileName: string; Separator: char);
var
   st:string;
   fich:textfile;
   i,j:Integer;
begin
   assignfile(fich,FileName);
   rewrite(fich);
   if Assigned(FOnLoad) then
      FOnLoad(self,0,self.items.count);
   for i:=0 to self.items.count-1 do
   begin
      if Assigned(FOnLoad) then
         FOnLoad(self,i+1,self.items.count);
      st:=self.items[i].Caption;
      if pos(Separator,st)<>0 then
        st:='"'+st+'"';
      for j:=0 to self.items[i].subitems.count-1 do
      begin
         if pos(Separator,self.items[i].subitems[j])=0 then
           st:=st+Separator+self.items[i].subitems[j]
         else
           st:=st+Separator+'"'+self.items[i].subitems[j]+'"';
      end;
      WriteLn(fich,st);
   end;
   CloseFile(fich);
end;
{**************************************************}
procedure TJvListView.InvertSelection;
var
   i:Integer;
begin
   items.BeginUpdate;
   for i:=0 to items.Count-1 do
      items[i].Selected:=not(items[i].Selected);
   items.EndUpdate;
end;
{**************************************************}
{$IFNDEF D6}
procedure TJvListView.SelectAll;
var
   i:Integer;
begin
   items.BeginUpdate;
   for i:=0 to items.Count-1 do
      items[i].Selected:=true;
   items.EndUpdate;
end;
{$ENDIF}
{**************************************************}
procedure TJvListView.UnselectAll;
var
   i:Integer;
begin
   items.BeginUpdate;
   for i:=0 to items.Count-1 do
      items[i].Selected:=false;
   items.EndUpdate;
end;
{**************************************************}
{$IFNDEF D6}
procedure TJvListView.DeleteSelected;
var
 i: Integer;
begin
  Items.BeginUpdate;
  if SelCount=1 then
  begin
    i := Selected.Index-1;
    Selected.Delete;
    if i=-1 then i:=0;
    if Items.Count>0 then
      Selected := Items[i];
  end
  else
  begin
   for i:=Items.Count-1 downto 0 do
     if Items[i].Selected then
       Items[i].Delete;
  end;
  Items.EndUpdate;
end;
{$ENDIF}
{**************************************************}
end.
