{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvListbox2.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvListbox2;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, StdCtrls, Controls, Forms,
  JvItemsSearchs;

type
  TListboxChange = procedure(Sender: TObject; Item: string) of object;

  TJvListbox2 = class(TListbox)
  private
    FEffect: boolean;
    FOnMouseEnter: TNotifyEvent;
    FColor,FSaved: TColor;
    FOnMouseLeave: TNotifyEvent;
    FOnCtl3DChanged: TNotifyEvent;
    FOnParentColorChanged: TNotifyEvent;
    FOnSelectCancel: TNotifyEvent;
    Fover: boolean;
    FMaxWidth: Integer;
    FScroll: boolean;
    FOnDelete: TListboxChange;
    FOnAdd: TListboxChange;
    FOnChange: TNotifyEvent;
    FOnHScroll: TNotifyEvent;
    FOnVScroll: TNotifyEvent;
    FItemSearchs: TJvItemsSearchs; 
    procedure SetHScroll(const Value: boolean);
    procedure RefreshH;
    procedure SetEffect(const Value: boolean);
  protected
    procedure LBAddString(var Message: TMessage); message LB_ADDstring;
    procedure LBDeleteString(var Message: TMessage); message LB_DELETEstring;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorCHANGED;
    procedure SelectCancel(var Message: TMessage);message LBN_SELCANCEL;
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
    procedure Changed;virtual;
  public
    procedure WndProc(var Message: TMessage); override;
    procedure CreateParams(var Params: TCreateParams); override;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;

    function SearchExactString(Value: string; CaseSensitive: Boolean = true):Integer;
    function SearchPrefix(Value: string; CaseSensitive: Boolean = true):Integer;
    function SearchSubString(Value: string; CaseSensitive: Boolean = true):Integer;
    function DeleteExactString(Value: string; All: Boolean;
      CaseSensitive: Boolean = true):Integer;
    procedure SelectAll;
    procedure UnselectAll;
    procedure InvertSelection;
    procedure MoveSelectedUp;virtual;
    procedure MoveSelectedDown;virtual;
    procedure DeleteSelected;
    procedure DeleteAllButSelected;
  published
    property HorScrollbar:boolean read FScroll write SetHScroll default true;
    property HotTrack: boolean read FEffect write SetEffect default false;
    property HintColor:TColor read FColor write FColor default clInfoBk;

    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged : TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
    property OnSelectCancel : TNotifyEvent read FOnSelectCancel write FOnSelectCancel;
    property OnChange:TNotifyEvent read FOnChange write FOnChange;
    property OnDeleteString:TListboxChange read FOnDelete write FOnDelete;
    property OnAddString:TListboxChange read FOnAdd write FOnAdd;
    property OnVerticalScroll:TNotifyEvent read FOnVScroll write FOnVScroll;
    property OnHorizontalScroll:TNotifyEvent read FOnHScroll write FOnHScroll;
  end;

implementation

{**************************************************}
constructor TJvListBox2.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FMaxWidth := 0;
  FEffect := false;
  FColor := clInfoBk;
  FOver := false;
  FScroll := true;
  Controlstyle := Controlstyle + [csAcceptsControls];
  FItemSearchs := TJvItemsSearchs.Create;
end;
{**************************************************}
destructor TJvListbox2.Destroy;
begin
  FItemSearchs.Free;
  inherited;
end;
{**************************************************}
procedure TJvListBox2.CreateParams(var Params: TCreateParams);
begin
  inherited;
  with Params do
    if FScroll then
      Style := Style or WS_HSCROLL
    else
      Style := Style xor WS_HSCROLL;
end;
{***********************************************}
procedure TJvListbox2.SetEffect(const Value: boolean);
begin
  FEffect := Value;
  if Feffect then
    ctl3d := false;
end;
{***********************************************}
procedure TJvListbox2.RefreshH;
var
 i: Integer;
 ItemWidth: Word;
begin
  FMaxWidth := 0;
  for i:=0 to self.Items.Count-1 do
  begin
    ItemWidth := Canvas.TextWidth(Items[I]+' ');
    if FMaxWidth<ItemWidth then
      FMaxWidth := ItemWidth;
  end;
  SetHScroll(FScroll);
end;
{***********************************************}
procedure TJvListbox2.LBAddString(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnAdd) then
    FOnAdd(self,StrPas(PChar(Message.lParam)));
  if Assigned(FOnChange) then
    FonChange(self);
end;
{***********************************************}
procedure TJvListbox2.LBDeleteString(var Message: TMessage);
begin
  if Assigned(FOnDelete) then
    FOnDelete(self,Items.strings[LongInt(Message.wParam)]);
  inherited;
  if Assigned(FonChange) then
    FonChange(self);
end;
{***********************************************}
procedure TJvListbox2.WndProc(var Message: TMessage);
var
 ItemWidth:Word;
begin
  case Message.Msg of
    LB_ADDstring, LB_INSERTSTRING :
       begin
         ItemWidth := Canvas.TextWidth(StrPas(PChar(Message.lParam))+' ');
         if FMaxWidth < ItemWidth then
           FMaxWidth := ItemWidth;
         SendMessage(Handle, LB_SETHORIZONTALEXTENT, FMaxWidth, 0);
       end;
    LB_DELETESTRING :
       begin
         ItemWidth:=Canvas.TextWidth(Items[Message.wParam]+' ');
         if ItemWidth = FMaxWidth then
         begin
           inherited;
           RefreshH;
           Exit;
         end;
       end;
    LB_RESETCONTENT : SendMessage(Handle, LB_SETHORIZONTALEXTENT, 0, 0);
    WM_SETFONT :
       begin
         inherited;
         Canvas.Font.Assign(Self.Font);
         RefreshH;
         Exit;
       end;
  end;
  inherited;
end;
{***********************************************}
procedure TJvListbox2.SetHScroll(const Value: boolean);
begin
  if FScroll<>Value then
  begin
    FScroll:=Value;
    if FScroll then
      SendMessage(Handle, LB_SETHORIZONTALEXTENT, FMaxWidth, 0);
    ReCreateWnd;
  end;
end;
{***********************************************}
procedure TJvListbox2.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvListbox2.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvListbox2.CMMouseEnter(var Message: TMessage);
begin
  if not FOver then
  begin
    FSaved := Application.HintColor;
    Application.HintColor := FColor;
    if FEffect then
      ctl3d := true;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvListbox2.CMMouseLeave(var Message: TMessage);
begin
  if FOver then
  begin
    Application.HintColor := FSaved;
    if FEffect then
      ctl3d := false;
    FOver := false;
  end;
  if Assigned(FonMouseLeave) then
    FonMouseLeave(self);
end;
{**************************************************}
function TJvListbox2.SearchExactString(Value: string;
  CaseSensitive: Boolean): Integer;
begin
  result := FItemSearchs.SearchExactString(Items,Value,CaseSensitive);
end;
{**************************************************}
function TJvListbox2.SearchPrefix(Value: string;
  CaseSensitive: Boolean):Integer;
begin
  result := FItemSearchs.SearchPrefix(Items,Value,CaseSensitive);
end;
{**************************************************}
procedure TJvListbox2.WMHScroll(var Message: TWMHScroll);
begin
  inherited;
  if Assigned(FOnHScroll) then
    FOnHScroll(self);
end;
{**************************************************}
procedure TJvListbox2.WMVScroll(var Message: TWMVScroll);
begin
  inherited;
  if Assigned(FOnVScroll) then
    FOnVScroll(self);
end;
{**************************************************}
procedure TJvListbox2.SelectCancel(var Message:TMessage);
begin
  if Assigned(FOnSelectCancel) then
    FOnSelectCancel(self);
end;
{**************************************************}
function TJvListbox2.SearchSubString(Value: string;
  CaseSensitive: Boolean): Integer;
begin
  result := FItemSearchs.SearchSubString(Items,Value,CaseSensitive);
end;
{**************************************************}
function TJvListbox2.DeleteExactString(Value: string; All: Boolean;
  CaseSensitive: Boolean): Integer;
begin
  result := FItemSearchs.DeleteExactString(Items,Value,CaseSensitive);
  Changed;
end;
{**************************************************}
procedure TJvListbox2.SelectAll;
var
 i: Integer;
begin
  if MultiSelect = false then
    Exit;
  Items.BeginUpdate;
  for i:=0 to Items.Count-1 do
    Selected[i] := true;
  Items.EndUpdate;
end;
{**************************************************}
procedure TJvListbox2.UnselectAll;
var
 i:Integer;
begin
  if MultiSelect = false then
    Exit;
  Items.BeginUpdate;
  for i:=0 to Items.Count-1 do
    Selected[i] := false;
  Items.EndUpdate;
end;
{**************************************************}
procedure TJvListbox2.InvertSelection;
var
 i: Integer;
begin
  if MultiSelect = false then
    Exit;
  Items.BeginUpdate;
  for i:=0 to Items.Count-1 do
     Selected[i] := not(Selected[i]);
  Items.EndUpdate;
end;
{**************************************************}
procedure TJvListbox2.DeleteSelected;
var
 i: Integer;
begin
  if MultiSelect then
  begin
    for i:=Items.Count-1 downto 0 do
     if Selected[i] then
       Items.Delete(i);
  end
  else
    if ItemIndex<>-1 then
    begin
      i := ItemIndex;
      Items.Delete(i);
      if i>0 then dec(i);
      if Items.Count>0 then
        ItemIndex := i;
    end;
  Changed;
end;
{**************************************************}
procedure TJvListbox2.MoveSelectedDown;
var
 i: Integer;
begin
  if MultiSelect = false then
  begin
    if (ItemIndex<>-1) and (ItemIndex<Items.Count-1) then
    begin
      Items.Exchange(ItemIndex, ItemIndex+1);
      ItemIndex := ItemIndex+1;
    end;
    Exit;
  end;
  if (Items.Count>0) and (SelCount>0) and (not(Selected[Items.Count-1])) then
  begin
    i := Items.Count-2;
    while i>=0 do
    begin
      if Selected[i] then
      begin
        Items.Exchange(i,i+1);
        Selected[i+1] := true;
      end;
      dec(i);
    end;
  end;
end;
{**************************************************}
procedure TJvListbox2.MoveSelectedUp;
var
 i:Integer;
begin
  if MultiSelect = false then
  begin
    if (ItemIndex>1) then
    begin
      Items.Exchange(ItemIndex, ItemIndex-1);
      ItemIndex := ItemIndex-1;
    end;
    Exit;
  end;
  if (Items.Count>0) and (SelCount>0)and(not(Selected[0])) then
  begin
    i := 1;
    while i<Items.Count do
    begin
      if Selected[i] then
      begin
        Items.Exchange(i,i-1);
        Selected[i-1] := true;
      end;
      inc(i);
    end;
  end;
end;
{**************************************************}
procedure TJvListbox2.DeleteAllButSelected;
var
 i: Integer;
begin
  if MultiSelect = false then
    Exit;
  i:=0;
  while i<Items.Count do
    if not(Selected[i]) then
      Items.Delete(i)
    else
      inc(i);
  Changed;
end;
{**************************************************}
procedure TJvListbox2.Changed;
begin

end;
{**************************************************}
end.
