{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvRadioGroup.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvRadioGroup;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, extctrls, JvPropAutoSave;

type
  TJvRadioGroup = class(TRadioGroup)
  private
    FOnMouseEnter: TNotifyEvent;
    FColor,FSaved: TColor;
    FOnMouseLeave: TNotifyEvent;
    FOnCtl3DChanged: TNotifyEvent;
    FOnParentColorChanged: TNotifyEvent;
    FOnRestored: TNotifyEvent;
    FOver: Boolean;
    FAutoSave: TJvAutoSave;
  protected
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorCHANGED;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  public
    destructor Destroy;override;
    constructor Create(AOwner : TComponent);override;
    procedure Loaded;override;
  published
    property AutoSave: TJvAutoSave read FAutoSave write FAutoSave;
    property HintColor:TColor read FColor write FColor default clInfoBk;

    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged : TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
    property OnRestored:TNotifyEvent read FOnRestored write FOnRestored;
  end;

implementation

{**************************************************}
constructor TJvRadioGroup.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FColor:=clInfoBk;
  Controlstyle := Controlstyle + [csAcceptsControls];
  FOver := false;
  FAutoSave := TJvAutoSave.Create(self);
end;
{***********************************************}
destructor TJvRadioGroup.Destroy;
begin
  FAutoSave.Free;  
  inherited;
end;
{***********************************************}
procedure TJvRadioGroup.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvRadioGroup.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvRadioGroup.CMMouseEnter(var Message: TMessage);
begin
  if not FOver then
  begin
    FSaved := Application.HintColor;
    Application.HintColor := FColor;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvRadioGroup.CMMouseLeave(var Message: TMessage);
begin
  if FOver then
  begin
    Application.HintColor:=FSaved;
    FOver := false;
  end;
  if Assigned(FonMouseLeave) then
    FonMouseLeave(self);
end;
{**************************************************}
procedure TJvRadioGroup.Loaded;
var
 i: Integer;
begin
  inherited;
  if (FAutoSave.LoadValue(i))  then
  begin
    ItemIndex := i;
    if Assigned(FOnRestored) then
      FOnRestored(self);
  end;
end;
{**************************************************}
procedure TJvRadioGroup.CMExit(var Message: TCMExit);
begin
  inherited;
  FAutoSave.SaveValue(ItemIndex);
end;
{**************************************************}
end.
