{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvRegistryViewer.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvRegistryViewer;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, JvTypes, comctrls, extctrls, registry, Menus;

type
  TJvRegistryViewer = class(TWinControl)
  private
     FTreeView1 : TTreeView;
     FSplitter1 : TSplitter;
     FListView1 : TListView;
     FRegistry:TRegistry;
     FChangeKey: TOnChangeKey;

     function GetTreeView1Width:Integer;
     procedure SetTreeView1Width(Value:integer);
     function GetTreeView1Cursor:TCursor;
     procedure SetTreeView1Cursor(Value:TCursor);
     function GetTreeView1Hint:string;
     procedure SetTreeView1Hint(Value:string);
     function GetListView1Width:Integer;
     procedure SetListView1Width(Value:integer);
     function GetListView1Hint:string;
     procedure SetListView1Hint(Value:string);
     procedure AddChild(Pere: TTreeNode; path: string;h:HKEY);
     procedure Expand(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
     procedure Collapse(Sender: TObject; Node: TTreeNode;var AllowCollapse: Boolean);
     procedure Changing(Sender: TObject; Node: TTreeNode;var AllowChange: Boolean);
    function GetPopupList: TPopupMenu;
    function GetPopupTree: TPopupMenu;
    procedure SetPopupList(const Value: TPopupMenu);
    procedure SetPopupTree(const Value: TPopupMenu);
  protected
     procedure Loaded;override;
  public
     constructor Create(AOwner: TComponent);override;
     destructor Destroy;override;
  published
     property TreeViewWidth:Integer read GetTreeView1Width write SetTreeView1Width;
     property TreeViewCursor:TCursor read GetTreeView1Cursor write SetTreeView1Cursor;
     property TreeViewHint:string read GetTreeView1Hint write SetTreeView1Hint;
     property ListViewWidth:Integer read GetListView1Width write SetListView1Width;
     property ListViewHint:string read GetListView1Hint write SetListView1Hint;
     property BorderWidth;
     property Align;
     property OnChangeKey:TOnChangeKey read FChangeKey write FChangeKey;
     property PopupTreeView:TPopupMenu read GetPopupTree write SetPopupTree;
     property PopupListView:TPopupMenu read GetPopupList write SetPopupList;
  end;

implementation

resourcestring
   RC_HKey1            =     'HKEY_CLASSES_ROOT';
   RC_HKey2            =     'HKEY_CURRENT_USER';
   RC_HKey3            =     'HKEY_LOCAL_MACHINE';
   RC_HKey4            =     'HKEY_USERS';
   RC_HKey5            =     'HKEY_PERFORMANCE_DATA';
   RC_HKey6            =     'HKEY_CURRENT_CONFIG';
   RC_HKey7            =     'HKEY_DYN_DATA';

   RC_DefaultCaption   =     '<Default>';
   RC_Unknow           =     'Unknow';
   RC_String           =     'String';
   RC_ExpandString     =     'ExpandString';
   RC_Integer          =     'Integer';
   RC_Binary           =     'Binary';

{***************************************************}
constructor TJvRegistryViewer.Create(AOwner: TComponent);
var
   t:TListColumn;
begin
   inherited Create(AOwner);

   self.width:=576;
   self.height:=358;
   FTreeView1:=TTreeView.Create(self);
   FTreeView1.parent:=self;
   FSplitter1:=TSplitter.Create(self);
   FSplitter1.parent:=self;
   FListView1:=TListView.Create(self);
   FListView1.parent:=self;

   FTreeView1.Left:=0;
   FTreeView1.Top:=0;
   FTreeView1.Width:=201;
   FTreeView1.Height:=331;
   FTreeView1.Align:=alLeft;

   FSplitter1.Left:=201;
   FSplitter1.Top:=0;
   FSplitter1.Width:=4;
   FSplitter1.Height:=331;
   FSplitter1.align:=alLeft;

   FListView1.Left:=205;
   FListView1.Top:=0;
   FListView1.Width:=363;
   FListView1.Height:=331;
   FListview1.Align:=AlClient;
   FListview1.Viewstyle:=vsReport;
   t:=FListview1.Columns.Add;
   t.Caption:='Name';
   t.Width:=200;
   t:=Flistview1.columns.add;
   t.caption:='Type';
   t.width:=50;
   t:=Flistview1.columns.add;
   t.caption:='Data';
   t.width:=100;

   FRegistry:=TRegistry.Create;
   FTreeView1.OnExpanding:=Expand;
   FTreeView1.OnCollapsing:=Collapse;
   FTreeView1.ReadOnly:=true;
   FTreeView1.OnChanging:=Changing;
   FListView1.ReadOnly:=true;
   FListView1.HotTrack:=true;
   FListView1.FlatScrollBars:=true;
   FListView1.RowSelect:=true;
   FListView1.FullDrag:=false;
   FListView1.ColumnClick:=false;
end;
{***************************************************}
destructor TJvRegistryViewer.Destroy;
begin
   FTreeView1.free;
   FSplitter1.free;
   FListView1.free;
   FRegistry.free;

   inherited Destroy;
end;
{***************************************************}
function TJvRegistryViewer.GetTreeView1Width:Integer;
begin
   result:=FTreeView1.Width;
end;
{***************************************************}
procedure TJvRegistryViewer.SetTreeView1Width(Value:integer);
begin
   FTreeView1.Width:=Value;
end;
{***************************************************}
function TJvRegistryViewer.GetTreeView1Cursor:TCursor;
begin
   result:=FTreeView1.Cursor;
end;
{***************************************************}
procedure TJvRegistryViewer.SetTreeView1Cursor(Value:TCursor);
begin
   FTreeView1.Cursor:=Value;
end;
{***************************************************}
function TJvRegistryViewer.GetTreeView1Hint:string;
begin
   result:=FTreeView1.Hint;
end;
{***************************************************}
procedure TJvRegistryViewer.SetTreeView1Hint(Value:string);
begin
   FTreeView1.Hint:=Value;
end;
{***************************************************}
function TJvRegistryViewer.GetListView1Width:Integer;
begin
   result:=FListView1.Width;
end;
{***************************************************}
procedure TJvRegistryViewer.SetListView1Width(Value:integer);
begin
   FListView1.Width:=Value;
end;
{***************************************************}
function TJvRegistryViewer.GetListView1Hint:string;
begin
   result:=FListView1.Hint;
end;
{***************************************************}
procedure TJvRegistryViewer.SetListView1Hint(Value:string);
begin
   FListView1.Hint:=Value;
end;
{***************************************************}
procedure TJvRegistryViewer.AddChild(Pere:TTreeNode;path:string;h:HKEY);
var
   tl:TTreeNode;
   ts:TstringList;
   i:Integer;
begin
   FRegistry.RootKey:=h;
   FRegistry.OpenKey(path,false);
   ts:=TstringList.Create;
   FRegistry.GetKeyNames(ts);
   for i:=0 to ts.count-1 do
   begin
      tl:=FTreeView1.Items.AddChild(Pere,ts[i]);
      AddChild(tl,path+ts[i]+'\',h);
   end;
   ts.free;
end;
{***************************************************}
procedure TJvRegistryViewer.Loaded;
var
   tl:TTreeNode;
begin
   FRegistry.rootkey:=HKEY_CLASSES_ROOT;
   FRegistry.OpenKey('\',false);
   tl:=FTreeview1.items.AddChildFirst(nil,RC_HKey1);
   if FRegistry.HasSubKeys then tl.HasChildren:=true;

   FRegistry.rootkey:=HKEY_CURRENT_USER;
   FRegistry.OpenKey('\',false);
   tl:=FTreeview1.items.AddChildFirst(nil,RC_HKey2);
   if FRegistry.HasSubKeys then tl.HasChildren:=true;

   FRegistry.rootkey:=HKEY_LOCAL_MACHINE;
   FRegistry.OpenKey('\',false);
   tl:=FTreeview1.items.AddChildFirst(nil,RC_HKey3);
   if FRegistry.HasSubKeys then tl.HasChildren:=true;

   FRegistry.rootkey:=HKEY_USERS;
   FRegistry.OpenKey('\',false);
   tl:=FTreeview1.items.AddChildFirst(nil,RC_HKey4);
   if FRegistry.HasSubKeys then tl.HasChildren:=true;

   FRegistry.rootkey:=HKEY_PERFORMANCE_DATA;
   FRegistry.OpenKey('\',false);
   tl:=FTreeview1.items.AddChildFirst(nil,RC_HKey5);
   if FRegistry.HasSubKeys then tl.HasChildren:=true;

   FRegistry.rootkey:=HKEY_CURRENT_CONFIG;
   FRegistry.OpenKey('\',false);
   tl:=FTreeview1.items.AddChildFirst(nil,RC_HKey6);
   if FRegistry.HasSubKeys then tl.HasChildren:=true;

   FRegistry.rootkey:=HKEY_DYN_DATA;
   FRegistry.OpenKey('\',false);
   tl:=FTreeview1.items.AddChildFirst(nil,RC_HKey7);
   if FRegistry.HasSubKeys then tl.HasChildren:=true;
end;
{***************************************************}
procedure TJvRegistryViewer.Expand(Sender: TObject; Node: TTreeNode;
     var AllowExpansion: Boolean);
var
   path:string;
   rootk:HKey;
   t,r:TTreeNode;
   ts:TstringList;
   i:Integer;
begin
   path:='\';
   t:=Node;
   r:=t;
   while t<>nil do
   begin
      if t.parent<>nil then
         path:='\'+t.Text+path;
      r:=t;
      t:=t.Parent;
   end;

   if r.Text=RC_HKey2 then rootk:=HKEY_CURRENT_USER
   else if r.text=RC_HKey1 then rootk:=HKEY_CLASSES_ROOT
   else if r.text=RC_HKey3 then rootk:=HKEY_LOCAL_MACHINE
   else if r.text=RC_HKey4 then rootk:=HKEY_USERS
   else if r.text=RC_HKey5 then rootk:=HKEY_PERFORMANCE_DATA
   else if r.text=RC_HKey7 then rootk:=HKEY_DYN_DATA
   else Rootk:=HKEY_CURRENT_CONFIG;

   FRegistry.rootkey:=rootk;

   Fregistry.OpenKey(path,false);
   ts:=TstringList.Create;
   FRegistry.GetKeyNames(ts);
   FRegistry.CloseKey;

   ts.Sort;

   for i:=0 to ts.count-1 do
   begin
      t:=FTreeview1.Items.AddChild(Node,ts[i]);
      t.text:=ts[i];
      FRegistry.OpenKey(path+ts[i]+'\',false);
      t.HasChildren:=FRegistry.HasSubKeys;
      FRegistry.CloseKey;
   end;

   AllowExpansion:=true;

   ts.free;
end;
{***************************************************}
procedure TJvRegistryViewer.Collapse(Sender: TObject; Node: TTreeNode;
  var AllowCollapse: Boolean);
begin
   AllowCollapse:=true;
   Node.DeleteChildren;
   Node.HasChildren:=true;
end;
{***************************************************}
procedure TJvRegistryViewer.Changing(Sender: TObject; Node: TTreeNode;
  var AllowChange: Boolean);
var
   path,st:string;
   rootk:HKey;
   t,r:TTreeNode;
   tl:TListItem;
   ts:TstringList;
   i,j,k:Integer;
   p : array [0..255] of byte;
begin
   self.FListView1.Items.Clear;

   path:='\';
   t:=Node;
   r:=t;
   while t<>nil do
   begin
      if t.parent<>nil then
         path:='\'+t.Text+path;
      r:=t;
      t:=t.Parent;
   end;
   if r.Text=RC_HKey2 then rootk:=HKEY_CURRENT_USER
   else if r.text=RC_HKey1 then rootk:=HKEY_CLASSES_ROOT
   else if r.text=RC_HKey3 then rootk:=HKEY_LOCAL_MACHINE
   else if r.text=RC_HKey4 then rootk:=HKEY_USERS
   else if r.text=RC_HKey5 then rootk:=HKEY_PERFORMANCE_DATA
   else if r.text=RC_HKey7 then rootk:=HKEY_DYN_DATA
   else Rootk:=HKEY_CURRENT_CONFIG;

   FRegistry.rootkey:=rootk;

   Fregistry.OpenKey(path,false);
   ts:=TstringList.Create;
   FRegistry.GetValueNames(ts);

   ts.Sort;

   for i:=0 to ts.count-1 do
   begin
      tl:=self.FListView1.Items.Add;
      if ts[i]='' then
         tl.caption:=RC_DefaultCaption
      else
         tl.Caption:=ts[i];
      case FRegistry.GetDataType(ts[i]) of
           rdUnknown : tl.Subitems.add(RC_Unknow);
           rdstring : begin
                         tl.Subitems.add(RC_string);
                         tl.Subitems.add(FRegistry.ReadString(ts[i]));
                      end;
           rdExpandstring : begin
                               tl.Subitems.add(RC_Expandstring);
                               tl.Subitems.add(FRegistry.ReadString(ts[i]));
                            end;
           rdInteger :  begin
                           tl.Subitems.add(RC_integer);
                           tl.Subitems.add(IntToStr(FRegistry.ReadInteger(ts[i])));
                        end;
           rdBinary : begin
                         tl.Subitems.add(RC_Binary);
                         j:=FRegistry.ReadBinaryData(ts[i],p,256);
                         st:='';
                         if j<>-1 then
                              for k:=0 to j-1 do
                                   st:=st+IntToHex(p[k],2)+' ';
                         tl.subitems.add(st);
                      end;
           else tl.Subitems.add(RC_Unknow);
       end;
   end;

   FRegistry.CloseKey;
   AllowChange:=true;

   if Assigned (FChangeKey) then
      FChangeKey(self,Rootk,path);

   ts.free;
end;
{***************************************************}
function TJvRegistryViewer.GetPopupList: TPopupMenu;
begin
   result:=FListView1.PopupMenu;
end;
{***************************************************}
function TJvRegistryViewer.GetPopupTree: TPopupMenu;
begin
   result:=FTreeView1.PopupMenu;
end;
{***************************************************}
procedure TJvRegistryViewer.SetPopupList(const Value: TPopupMenu);
begin
   FListView1.PopupMenu:=Value;
end;
{***************************************************}
procedure TJvRegistryViewer.SetPopupTree(const Value: TPopupMenu);
begin
   FTreeView1.PopupMenu:=Value;
end;
{***************************************************}
end.
