{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvSpeedButton.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvSpeedButton;

interface

{$ObjExportAll On}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, Menus, comctrls;

type
  TJvSpeedButton = class(TSpeedButton)
  private
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FOnParentColorChanged: TNotifyEvent;
    FSaved,Fcolor: TColor;
    FOver: boolean;
    FGlyph: TBitmap;
    FOldGlyph: TBitmap;
    FDropDown: TPopupMenu;
    FModalResult: TModalResult;
    FHotTrack: boolean;
    FHotFont: TFont;
    FFontSave: TFont;
    procedure SetGlyph(Value: TBitmap);
    procedure SetHotFont(const Value: TFont);
  protected
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorCHANGED;
  public
    procedure Click; override;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property HotTrack: boolean read FHotTrack write FHotTrack default false;
    property HotTrackFont: TFont read FHotFont write SetHotFont;
    property HotGlyph:TBitmap read FGlyph write SetGlyph;
    property HintColor:TColor read FColor write FColor default clInfoBk;
    property DropDownMenu:TPopupMenu read FDropDown write FDropDown;
    property ModalResult: TModalResult read FModalResult write FModalResult default mrNone;

    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
  end;

implementation

{**************************************************}
constructor TJvSpeedButton.Create(AOwner: TComponent);
begin
  inherited;
  FHotTrack := false;
  FHotFont := TFont.Create;
  FFontSave := TFont.Create;
  FColor := clInfoBk;
  FOver := false;
  FGlyph := TBitmap.Create;
  FOldGlyph := TBitmap.Create;
  FModalResult := mrNone;
end;
{**************************************************}
procedure TJvSpeedButton.Click;
var
 Form: TForm;
begin
  inherited;
  if FModalResult<>mrNone then
  begin
    Form:=TForm(GetParentForm(self));
    if Form<>nil then Form.ModalResult:=FModalResult;
  end
  else if FDropDown<>nil then
  begin
     FDropDown.Popup(GetClientOrigin.x,GetClientOrigin.y+self.height);
     Perform(CM_MOUSELEAVE,0,0);
  end;
end;
{**************************************************}
destructor TJvSpeedButton.Destroy;
begin
  FGlyph.free;
  FOldGlyph.free;
  FHotFont.Free;
  FFontSave.Free;
  inherited;
end;
{**************************************************}
procedure TJvSpeedButton.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvSpeedButton.SetGlyph(Value: TBitmap);
begin
  FGlyph.Assign(Value);
end;
{**************************************************}
procedure TJvSpeedButton.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FOver then
  begin
    FSaved := Application.HintColor;
    Application.HintColor := FColor;
    if not(FGlyph.Empty) then
    begin
      FOldGlyph.Assign(Glyph);
      Glyph.Assign(FGlyph);
    end;
    if FHotTrack then
    begin
      FFontSave.Assign(Font);
      Font.Assign(FHotFont);
    end;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvSpeedButton.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FOver then
  begin
    Application.HintColor:=FSaved;
    if not(FOldGlyph.Empty) then
      Glyph.Assign(FOldGlyph);
    if FHotTrack then
      Font.Assign(FFontSave);
    FOver:=false;
  end;
  if Assigned(FonMouseLeave) then FonMouseLeave(self);
end;
{**************************************************}
procedure TJvSpeedButton.SetHotFont(const Value: TFont);
begin
  FHotFont.Assign(Value);
end;
{**************************************************}
end.
