{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvStaticText.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvStaticText;

interface

{$ObjExportAll On}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,  stdctrls, Forms;

type
  TJvStaticText = class(TStaticText)
  private
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    FOnCtl3DChanged:  TNotifyEvent;
    FOnParentColorChanged: TNotifyEvent;
    FColor,FSaved: TColor;
    FOver: boolean;
    FFontSave: TFont;
    FHotTrack: boolean;
    FHotFont: TFont;
    procedure SetHotFont(const Value: TFont);
  protected
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorChangeD;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property HotTrack: boolean read FHotTrack write FHotTrack default false;
    property HotTrackFont: TFont read FHotFont write SetHotFont;
    property HintColor:TColor read FColor write FColor default clInfoBk;

    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged : TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
  end;

implementation

{**************************************************}
constructor TJvStaticText.Create(AOwner: TComponent);
begin
  inherited;
  FColor := clInfoBk;
  FHotFont := TFont.Create;
  FFontSave := TFont.Create;
  FOver := false;
  Controlstyle := Controlstyle + [csAcceptsControls];
end;
{**************************************************}
procedure TJvStaticText.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvStaticText.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvStaticText.CMMouseEnter(var Message: TMessage);
begin
  if not FOver then
  begin
    FSaved := Application.HintColor;
    Application.HintColor := FColor;
    if FHotTrack then
    begin
      FFontSave.Assign(Font);
      Font.Assign(FHotFont);
    end;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvStaticText.CMMouseLeave(var Message: TMessage);
begin
  if FOver then
  begin
    Application.HintColor := FSaved;
    Font.Assign(FFontSave);
    FOver := false;
  end;
  if Assigned(FonMouseLeave) then
    FonMouseLeave(self);
end;
{**************************************************}
destructor TJvStaticText.Destroy;
begin
  FHotFont.Free;
  FFontSave.Free;
  inherited;
end;
{**************************************************}
procedure TJvStaticText.SetHotFont(const Value: TFont);
begin
  FHotFont.Assign(Value);
end;
{**************************************************}
end.
