{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvStringGrid.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvStringGrid;

interface

{$ObjExportAll On}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, grids,
  JvTypes;
  
type
  TJvSortType = (stAutomatic, stClassic, stCaseSensitive, stNumeric, stDate, stCurrency);
  {$EXTERNALSYM TJvSortType}
  TProgress = procedure (Sender: TObject; Progression,Total: integer) of object;

  TJvStringGrid = class(TStringGrid)
  private
    FOnMouseEnter : TNotifyEvent;
    FColor,FSaved:TColor;
    FOnMouseLeave : TNotifyEvent;
    FOnCtl3DChanged : TNotifyEvent;
    FOnParentColorChanged : TNotifyEvent;
    FOver:boolean;
    FOnLoad: TProgress;
    FOnSave: TProgress;
    FOnHScroll: TNotifyEvent;
    FOnVScroll: TNotifyEvent;
  protected
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorChangeD;
  public
    constructor Create(AOwner: TComponent);override;

    procedure SortGrid(Column: Integer; Ascending: boolean = true; Fixed: boolean = false;
      SortType: TJvSortType = stClassic; BlankTop: boolean = true);
    procedure SaveToFile(FileName: string);
    procedure LoadFromFile(FileName: string);
    procedure LoadFromCSV(FileName: string; Separator: Char = ';');
    procedure SaveToCSV(FileName: string; Separator: Char = ';');
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream: TStream);
  published
    property HintColor:TColor read FColor write FColor default clInfoBk;

    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged : TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
    property OnLoadProgress:TProgress read FOnLoad write FOnLoad;
    property OnSaveProgress:TProgress read FOnSave write FOnSave;
    property OnVerticalScroll:TNotifyEvent read FOnVScroll write FOnVScroll;
    property OnHorizontalScroll:TNotifyEvent read FOnHScroll write FOnHScroll;
  end;
  
implementation

{**************************************************}
procedure TJvstringGrid.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvstringGrid.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvstringGrid.SortGrid(Column: Integer; Ascending,
  Fixed: boolean;SortType:TJvSortType;BlankTop:boolean);
var
  st:string;
  tmpc:currency;
  tmpf:double;
  tmpd:TDateTime;
  debut,fin:Integer;

  procedure ExchangeGridRows(i,j:integer);
  var
    k:Integer;
  begin
     if Fixed then
        for k:=0 to ColCount-1 do
            Cols[k].Exchange(i,j)
     else
        for k:=FixedCols to ColCount-1 Do
            Cols[k].Exchange(i,j);
  end;

  function isSmaller(First,Second:string):boolean;
  var
    i,j:int64;
    f1,f2:double;
    c1,c2:currency;
  begin
     try
       i:=StrToInt64(First);
       j:=StrToInt64(Second);
       result:=i<j;
       exit;
     except
     end;
     try
       f1:=StrToFloat(First);
       f2:=StrToFloat(Second);
       result:=f1<f2;
       exit;
     except
     end;
     try
       c1:=StrToCurr(First);
       c2:=StrToCurr(Second);
       result:=c1<c2;
       exit;
     except
     end;
     result:=First>Second;
  end;

  function isBigger(First,Second:string):boolean;
  begin
     result:=isSmaller(Second,First);
  end;

  procedure QuickSort(l,r:integer);
  var
    i,j,m:Integer;
  begin
     repeat
        i:=l;
        j:=r;
        m:=(l+r) div 2;
        st:=Cells[Column,m];
        repeat
            case SortType of
                 stClassic       :
                     begin
                        while CompareText(Cells[Column,i],st)<0 do
                              inc(i);
                        while CompareText(Cells[Column,j],st)>0 do
                              dec(j);
                     end;
                 stCaseSensitive :
                     begin
                        while CompareStr(Cells[Column,i],st)<0 do
                              inc(i);
                        while CompareStr(Cells[Column,j],st)>0 do
                              dec(j);
                     end;
                 stNumeric       :
                     begin
                        tmpf:=StrToFloat(st);
                        while StrToFloat(Cells[Column,i])<tmpf do
                              inc(i);
                        while StrToFloat(Cells[Column,j])>tmpf do
                              dec(j);
                     end;
                 stDate          :
                     begin
                        tmpd:=StrToDateTime(st);
                        while StrToDateTime(Cells[Column,i])<tmpd do
                              inc(i);
                        while StrToDateTime(Cells[Column,j])>tmpd do
                              dec(j);
                     end;
                 stCurrency      :
                     begin
                        tmpc:=StrToCurr(st);
                        while StrToCurr(Cells[Column,i])<tmpc do
                              inc(i);
                        while StrToCurr(Cells[Column,j])>tmpc do
                              dec(j);
                     end;
                 stAutomatic     :
                     begin
                        while isSmaller(Cells[Column,i],st) do
                              inc(i);
                        while isBigger(Cells[Column,j],st) do
                              dec(j);
                     end;
            end;
            if i<=j then
            begin
               if i<>j then
                  ExchangeGridRows(i,j);
               inc(i);
               dec(j);
            end;
        until i>j;
        if l<j then QuickSort(l,j);
        l:=i;
     until i>=r;
  end;

  procedure InvertGrid;
  var
    i,j:Integer;
  Begin
     i:=Fixedrows;
     j:=Rowcount-1;
     while i<j do
     begin
       ExchangeGridRows(i,j);
       inc(i);dec(j);
     end;
  end;

  function MoveBlankTop:Integer;
  var
    i,j:Integer;
  Begin
     i:=Fixedrows;
     result:=i;
     j:=Rowcount-1;
     while i<=j do
     begin
       if trim(Cells[Column,i])='' then
       begin
         ExchangeGridRows(result,i);
         inc(result);
       end;
       inc(i);
     end;
  end;

begin
  if (Column>=0) and (Column<ColCount) then
  begin
    debut := FixedRows;
    fin := RowCount-1;
    if BlankTop then
       debut := MoveBlankTop;

    QuickSort(Debut,Fin);
    if not(Ascending) then
      InvertGrid;
  end;
end;
{**************************************************}
constructor TJvstringGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColor := clInfoBk;
  FOver := false;
  Controlstyle := Controlstyle + [csAcceptsControls];
end;
{**************************************************}
procedure TJvstringGrid.LoadFromFile(FileName: string);
var
 stream:TFileStream;
begin
  stream := TFileStream.Create(FileName,fmOpenRead or fmShareDenyWrite);
  LoadFromStream(Stream);
  stream.free;
end;
{**************************************************}
procedure TJvstringGrid.LoadFromCSV(FileName: string; Separator: char);
var
 st,st2: string;
 i,j,k,l,m,n: Integer;
 fich: TextFile;
 fpos,count: Integer;
 f: file of byte;
begin
  fpos:=0;
  AssignFile(f,FileName);
  reset(f);
  count:=FileSize(f);
  CloseFile(f);
  if Assigned(FOnLoad) then
     FOnLoad(self,fpos,count);

  AssignFile(fich,FileName);
  Reset(fich);
  k:=0;
  while not(eof(fich)) do
  begin
    ReadLn(fich,st);
    fpos:=fpos+length(st)+2;
    if Assigned(FOnLoad) then
      FOnLoad(self,fpos,count);

    //Analyse st
    j:=0;l:=1;
    for i:=1 to length(st) do
       if st[i]='"' then j:=(j+1)mod 2
       else if st[i]=Separator then if j=0 then inc(l);
    if ColCount<l then
       ColCount:=l;
    inc(k);
    if RowCount<k then RowCount:=k;

    j:=0;
    m:=pos(Separator,st);
    n:=pos('"',st);
    while m<>0 do
    begin
      if (n=0) or (n>m) then
      begin
        Cells[j,k-1]:=Copy(st,1,m-1);
        st := Copy(st,m+1,length(st));
      end
      else
      begin
        st := Copy(st,n+1,length(st));
        n := pos('"',st);
        st2 := Copy(st,1,n-1);
        st := Copy(st,n+1,length(st));
        m := pos(Separator,st);
        if m<>0 then
          st := Copy(st,m+1,length(st))
        else
          st := '';
        Cells[j,k-1] := st2;
      end;
      inc(j);

      m := pos(Separator,st);
      n := pos('"',st);
    end;
    if st<>'' then
      Cells[j,k-1]:=st;
  end;
  if Assigned(FOnLoad) then
    FOnLoad(self,count,count);
  CloseFile(fich);
end;
{**************************************************}
procedure TJvstringGrid.LoadFromStream(Stream: TStream);
var
 col,row,i,count: Integer;
 buf: array [0..1024] of byte;
 st: string;
begin
  col := 0;
  row := 1;
  if Assigned(FOnLoad) then
    FOnLoad(self,0,stream.Size);
  while stream.Position<stream.Size do
  begin
    count:=stream.Read(buf,1024);
    if Assigned(FOnLoad) then
      FOnLoad(self,stream.Position,stream.Size);
    for i:=0 to count-1 do
       case buf[i] of
         0 : begin
               inc(col);
               if row>RowCount then
                 RowCount:=row;
               if col>ColCount then
                 ColCount:=col;
               self.Cells[col-1,row-1] := st;
               st := '';
             end;
         1 : begin
               inc(col);
               if col>ColCount then
                 ColCount := col;
               Cells[col-1,row-1] := st;
               inc(row);
               if row>RowCount then
                 RowCount := Row;
               col := 0;
               st := '';
             end;
         else
           st := st+char(buf[i]);
       end;
  end;
  RowCount := RowCount-1;
  if Assigned(FOnLoad) then
    FOnLoad(self,Stream.Size,Stream.Size);
end;
{**************************************************}
procedure TJvstringGrid.WMHScroll(var Message: TWMHScroll);
begin
  inherited;
  if Assigned(FOnHScroll) then
     FOnHScroll(self);
end;
{**************************************************}
procedure TJvstringGrid.WMVScroll(var Message: TWMVScroll);
begin
  inherited;
  if Assigned(FOnVScroll) then
     FOnVScroll(self);
end;
{**************************************************}
procedure TJvstringGrid.CMMouseEnter(var Message: TMessage);
begin
  if not FOver then
  begin
    FSaved := Application.HintColor;
    Application.HintColor := FColor;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvstringGrid.CMMouseLeave(var Message: TMessage);
begin
  if FOver then
  begin
    Application.HintColor := FSaved;
    FOver := false;
  end;
  if Assigned(FonMouseLeave) then
    FonMouseLeave(self);
end;
{**************************************************}
procedure TJvstringGrid.SaveToFile(FileName: string);
var
 stream: TFileStream;
begin
  stream := TFileStream.Create(FileName,fmCreate or fmShareDenyWrite);
  SaveToStream(stream);
  stream.free;
end;
{**************************************************}
procedure TJvstringGrid.SaveToCSV(FileName: string; Separator: char);
var
 st: string;
 i,j: Integer;
 fich: TextFile;
begin
  AssignFile(fich,FileName);
  Rewrite(fich);
  if Assigned(FOnSave) then
    FOnSave(self,0,RowCount*ColCount);
  for i:=0 to RowCount-1 do
  begin
    st := '';
    for j:=0 to ColCount-1 do
    begin
      if Assigned(FOnSave) then
        FOnSave(self,i*ColCount+j,RowCount*ColCount);
      if pos(Separator,Cells[j,i])=0 then
        st := st+Cells[j,i]
      else
        st := st+'"'+Cells[j,i]+'"';
      if j<>ColCount-1 then
        st := st+Separator
    end;
    WriteLn(fich,st);
  end;
  CloseFile(fich);
  if Assigned(FOnSave) then
    FOnSave(self,RowCount*ColCount,RowCount*ColCount);
end;
{**************************************************}
procedure TJvstringGrid.SaveToStream(Stream: TStream);
var
 i,j,k: Integer;
 st: array [0..1000] of char;
 stt: string;
 a,b: byte;
begin
  a:=0;
  b:=1; //a for end of string, b for end of line
  if Assigned(FOnSave) then
    FOnSave(self,0,RowCount*ColCount);
  for i:=0 to RowCount-1 do
  begin
    for j:=0 to ColCount-1 do
    begin
      if Assigned(FOnSave) then
        FOnSave(self,i*ColCount+j,RowCount*ColCount);
      stt := Cells[j,i];
      for k:=1 to length(stt) do
        st[k-1]:=stt[k];
      stream.Write(st,length(Cells[j,i]));
      if j<>ColCount-1 then
        Stream.Write(a,1);
    end;
    Stream.Write(b,1);
  end;
  if Assigned(FOnSave) then
     FOnSave(self,RowCount*ColCount,RowCount*ColCount);
end;
{**************************************************}

end.
