{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvTipsOfDay.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvTipsOfDay;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  JvFormTips, JvBaseDlg, JvTypes, Registry, JvButtonPersistent;

type
  TJvTipsOfDay = class(TJvCommonDialogP)
  private
    FSCaption: string;
    FTitle: string;
    FDid: string;
    FColor: TColor;
    FHints: TstringList;
    FFont: TFont;
    FClose: TJvButtonPersistent;
    FNext: TJvButtonPersistent;
    procedure Setstrings(const Value: TstringList);
    procedure SetFont(const Value: TFont);
  protected
    procedure Loaded;override;
    function GetRegKey:string;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;

    function GetShowHintOnStartup:boolean;
    procedure SetShowHintOnStartup(Value:boolean);
    procedure ManuallyExecute;
    procedure Execute;override;
  published
    property Hints:TstringList read FHints write Setstrings;
    property Title:string read FTitle write Ftitle;
    property TextShowOnStartup:string read FSCaption write FSCaption;
    property TextDidYouKnow:string read FDid write FDid;
    property Font:TFont read FFont write SetFont;
    property Color:TColor read FColor write FColor default clInfoBk;

    property ButtonNext:TJvButtonPersistent read FNext write FNext;
    property ButtonClose:TJvButtonPersistent read FClose write FClose;
  end;

implementation

resourcestring
   RC_KeyStartup           =     'Software\BuyPin\TipsStartup';
   RC_TipsTitle            =     '&Show Tips on startup';
   RC_CloseCaption         =     '&Close';
   RC_TipsTitle2           =     'Tips and Tricks';
   RC_TipsLabel1           =     'Did you know...';
   RC_NextCaption          =     '&Next Tip';

{**************************************************}
constructor TJvTipsOfDay.Create(AOwner: TComponent);
begin
  inherited;
  FClose := TJvButtonPersistent.Create;
  FNext := TJvButtonPersistent.Create;
  FClose.Caption := RC_CloseCaption;
  FNext.Caption := RC_NextCaption;

  FHints:=TStringList.Create;
  FSCaption:=RC_TipsTitle;
  FTitle:=RC_TipsTitle2;
  FDid:=RC_TipsLabel1;
  FColor:=clInfoBk;
  FFont:=TFont.Create;
end;
{**************************************************}
destructor TJvTipsOfDay.Destroy;
begin
  FClose.Free;
  FNext.Free;
  FFont.Free;
  FHints.free;
  inherited;
end;
{**************************************************}
procedure TJvTipsOfDay.Execute;
begin
  ManuallyExecute;
end;
{**************************************************}
function TJvTipsOfDay.GetShowHintOnStartup: boolean;
begin
  result:=true;
  with TRegistry.Create do
  begin
    OpenKey(RC_KeyStartup,true);
    if ValueExists(GetRegKey) then
      result:=ReadBool(GetRegKey);
    Free;
  end;
end;
{**************************************************}
procedure TJvTipsOfDay.ManuallyExecute;
begin
  with TFormTip.Create(Application) do
  begin
    Caption:=Ftitle;
    did.caption:=FDid;
    did.Color:=FColor;
    tip.Color:=FColor;
    pnlTip.color:=FColor;
    tip.Font.Assign(FFont);
    ShowTips.caption:=FSCaption;

    FNext.AssignTo(BuSpeedButton1);
    FClose.AssignTo(BuSpeedButton2);

    if FHints.count>0 then
    begin
      it := Random(FHints.Count);
      if FHints[it]='' then it := 0;
      tip.Caption:=FHints[it];
      hints.Assign(FHints);
      ShowModal;
    end;

    SetShowHintOnStartup(showtips.Checked);
    Free;
  end;
end;
{**************************************************}
procedure TJvTipsOfDay.SetFont(const Value: TFont);
begin
  FFont.assign(Value);
end;
{**************************************************}
procedure TJvTipsOfDay.SetShowHintOnStartup(Value:boolean);
begin
  with TRegistry.Create do
  begin
    OpenKey(RC_KeyStartup,true);
    WriteBool(GetRegKey,Value);
    Free;
  end;
end;
{**************************************************}
procedure TJvTipsOfDay.SetStrings(const Value: TStringList);
begin
  FHints.Assign(Value);
end;
{**************************************************}
procedure TJvTipsOfDay.Loaded;
begin
  inherited;
  if GetShowHintOnStartup then
     ManuallyExecute;
end;
{**************************************************}
function TJvTipsOfDay.GetRegKey: string;
begin
  result := Application.Name+'_'+Name;
end;
{**************************************************}
end.
