{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvVigenereCipher.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvVigenereCipher;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, JvTypes;

type
  TJvVigenereCipher = class(TComponent)
  private
    FKey : string;
    FDecoded : string;
    FEncoded : string;
    procedure SetDecoded(st:string);
    procedure SetEncoded(st:string);
    function trans(Ch:Char;k:Byte):Char;
    function Decrypte(St:string;Ke:string):string;
    function Crypte(St:string;Ke:string):string;
  protected
  public
  published
    property Key : string read FKey write FKey;
    property Encoded : string read Fencoded write SetEncoded;
    property Decoded : string read FDecoded write SetDecoded;
    procedure Decode(It:Tstrings);
    procedure Encode(It:Tstrings);
    function EncodeStream(Value:TStream):TStream;
    function DecodeStream(Value:TStream):TStream;
  end;

implementation

{***********************************************************}
function TJvVigenereCipher.trans(Ch:Char;k:Byte):Char;
var
   i:Integer;
begin
   i:=ord(ch)+k;
   if i<0 then i:=i+256
   else if i>255 then i:=i-256;
   result:=char(i);
end;
{***********************************************************}
function TJvVigenereCipher.Decrypte(St:string;Ke:string):string;
var
  i,j:byte;
begin
   result:='';
   j:=1;
   if length(ke)>0 then
      for i:=1 to length(st) do
      begin
         result:=result+trans(st[i],-ord(ke[j]));
         if j=length(ke) then j:=1 else inc(j);
      end;
end;
{***********************************************************}
function TJvVigenereCipher.Crypte(St:string;Ke:string):string;
var
  i,j:byte;
begin
   result:='';
   j:=1;
   if length(ke)>0 then
      for i:=1 to length(st) do
      begin
         result:=result+trans(st[i],ord(ke[j]));
         if j=length(ke) then j:=1 else inc(j);
      end;
end;
{***********************************************************}
procedure TJvVigenereCipher.SetDecoded(st:string);
begin
   FDecoded:=st;
   FEncoded:=Crypte(st,FKey);
end;
{***********************************************************}
procedure TJvVigenereCipher.SetEncoded(st:string);
begin
   FEncoded:=st;
   FDecoded:=Decrypte(st,FKey);
end;
{***********************************************************}
procedure TJvVigenereCipher.Decode(It:Tstrings);
begin
   It.Text:=Decrypte(It.Text,FKey);
end;
{***********************************************************}
procedure TJvVigenereCipher.Encode(it:Tstrings);
begin
   It.Text:=Crypte(It.Text,FKey);
end;
{***********************************************************}
function TJvVigenereCipher.DecodeStream(Value: TStream): TStream;
var
  buffer:array [0..1024] of byte;
  i,j,count:Integer;
begin
   result:=TMemoryStream.Create;
   j:=1;
   while Value.Position<Value.Size do
   begin
      count:=Value.Read(buffer,1024);
      for i:=0 to count-1 do
      begin
         buffer[i]:=ord(Trans(char(buffer[i]),-ord(FKey[j])));
         if j=length(FKey) then j:=1 else inc(j);
      end;
      result.Write(buffer,count);
   end;
end;
{***********************************************************}
function TJvVigenereCipher.EncodeStream(Value: TStream): TStream;
var
  buffer:array [0..1024] of byte;
  i,j,count:Integer;
begin
   result:=TMemoryStream.Create;
   j:=1;
   while Value.Position<Value.Size do
   begin
      count:=Value.Read(buffer,1024);
      for i:=0 to count-1 do
      begin
         buffer[i]:=ord(Trans(char(buffer[i]),ord(FKey[j])));
         if j=length(FKey) then j:=1 else inc(j);
      end;
      result.Write(buffer,count);
   end;
end;
{***********************************************************}
end.
