{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvWaitingProgress.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvWaitingProgress;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, JvSpecialProgress;

type
  TJvProgressThread = class(TThread)
  private
  protected
    procedure Draw;
    procedure Execute; override;
  public
    FDelay:cardinal;
    FOnDraw:TNotifyEvent;
  end;

  TJvWaitingProgress = class(TWinControl)
  private
    FActive: boolean;
    FRefresh: cardinal;
    FLength: cardinal;
    FOnEnded: TNotifyEvent;
    FWait: TJvProgressThread;
    FProgress: TJvSpecialProgress;

    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure SetActive(const Value: boolean);
    procedure SetLength(const Value: cardinal);
    procedure SetRefresh(const Value: cardinal);
    procedure OnScroll(Sender : TObject);
    function GeTColor: TColor;
    procedure SeTColor(const Value: TColor);
    function GetBColor: TColor;
    procedure SetBColor(const Value: TColor);
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Length:cardinal read FLength write SetLength default 30000;
    property Active:boolean read FActive write SetActive default false;
    property RefreshInterval:cardinal read FRefresh write SetRefresh default 500;
    property OnEnded:TNotifyEvent read FOnEnded write FOnEnded;
    property ProgressColor:TColor read GeTColor write SeTColor;
    property Color:TColor read GetBColor write SetBColor;
  end;

implementation

{**************************************************}
constructor TJvWaitingProgress.Create(AOwner: TComponent);
begin
   inherited;
   FActive:=false;
   FRefresh:=500;
   FLength:=30000;

   FWait:=TJvProgressThread.Create(true);
   FWait.FreeOnTerminate:=false;
   FWait.FDelay:=FRefresh;
   FWait.FOnDraw:=OnScroll;

   FProgress:=TJvSpecialProgress.Create(self);
   FProgress.Parent:=self;
   FProgress.Maximum:=FLength;
   FProgress.Position:=0;
   FProgress.StarTColor:=clBlack;
   FProgress.EndColor:=clBlack;
   FProgress.Solid:=true;

   self.width:=100;
   self.height:=10;

   FProgress.left:=0;
   FProgress.top:=0;
   FProgress.Width:=self.width;
   FProgress.height:=self.height;

   inherited Color:=FProgress.Color;
end;
{**************************************************}
destructor TJvWaitingProgress.Destroy;
begin
   FWait.Terminate;
   while not(FWait.Terminated) do
      Application.ProcessMessages;
   FWait.Free;

   FProgress.Free;
   inherited;
end;
{**************************************************}
function TJvWaitingProgress.GetBColor: TColor;
begin
   result:=FProgress.Color;
end;
{**************************************************}
function TJvWaitingProgress.GeTColor: TColor;
begin
   result:=FProgress.StarTColor;
end;
{**************************************************}
procedure TJvWaitingProgress.OnScroll(Sender: TObject);
begin
  //Step
  if integer(FProgress.Position)+integer(FRefresh)>integer(FLength) then
  begin
     FProgress.Position:=FLength;
     SetActive(False);
     if Assigned(FOnEnded) then
        FOnEnded(self);
  end
  else
    FProgress.Position:=FProgress.Position+integer(FRefresh);
  Application.ProcessMessages;
end;
{**************************************************}
procedure TJvWaitingProgress.SetActive(const Value: boolean);
begin
   if FActive<>Value then
   begin
      FActive := Value;
      if (FActive) then
      begin
         FProgress.Position:=0;
         FWait.Resume;
      end
      else
         FWait.Suspend;
   end;
end;
{**************************************************}
procedure TJvWaitingProgress.SetBColor(const Value: TColor);
begin
   if FProgress.Color<>Value then
   begin
      FProgress.Color:=Value;
      inherited Color:=Value;
   end;
end;
{**************************************************}
procedure TJvWaitingProgress.SeTColor(const Value: TColor);
begin
   FProgress.StarTColor:=Value;
   FProgress.EndColor:=Value;
end;
{**************************************************}
procedure TJvWaitingProgress.SetLength(const Value: cardinal);
begin
   FLength := Value;
   FProgress.Position:=0;
   FProgress.Maximum:=FLength;
end;
{**************************************************}
procedure TJvWaitingProgress.SetRefresh(const Value: cardinal);
begin
   FRefresh := Value;
   FWait.FDelay:=FRefresh;
end;
{**************************************************}
procedure TJvWaitingProgress.WMSize(var Message: TWMSize);
begin
   FProgress.width:=self.width;
   FProgress.height:=self.height;
end;
{**************************************************}

///////////////////////////////////////////////////////////
// TJvProgressThread
///////////////////////////////////////////////////////////

{**************************************************}
procedure TJvProgressThread.Draw;
begin
   if Assigned(FOnDraw) then
     FOnDraw(nil)
end;
{**************************************************}
procedure TJvProgressThread.Execute;
begin
   while not(Terminated) do
   begin
      SynChronize(Draw);
      sleep(FDelay);
   end;
end;
{**************************************************}

end.
