{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvWinampApi.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvWinampApi;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, 
  JvHWinamp, JvTypes;

type
  TJvWinampApi = class(TComponent)
  private
    FBidon: boolean;
    FBidoni:Integer;
    FBidonW:TWStatus;
    FBidonT: TTime;
    function GetWinampPresent: boolean;
    function GetMajorVersion: Integer;
    function GetMinorVersion: Integer;
    function GetWStatus: TWStatus;
    function GetLength: TTime;
    function GeTTime: TTime;
    procedure SeTTime(const Value: TTime);
    function GetPos: Integer;
    procedure SetPos(const Value: integer);
    function GetBitRate: Integer;
    function GetChannels: Integer;
    function GetSampleRate: Integer;
    function GetListLength: Integer;
    function WinampHigher(Major,Minor:integer):boolean;
    procedure SendCommand(Value:integer);
  protected
  public
  published
    property WinampPresent:boolean read GetWinampPresent write FBidon;
    property WinampMajorVersion:Integer read GetMajorVersion write FBidoni;
    property WinampMinorVersion:Integer read GetMinorVersion write FBidoni;
    property WinampStatus:TWStatus read GetWStatus write FBidonW;
    property SongPosition:TTime read GeTTime write SeTTime;
    property SongLength:TTime read GetLength write FBidonT;
    property PlaylistPos:Integer read GetPos write SetPos;
    property SampleRate:Integer read GetSampleRate write FBidoni;
    property BitRate:Integer read GetBitRate write FBidoni;
    property Channels:Integer read GetChannels write FBidoni;
    property ListLength:Integer read GetListLength write FBidonI;
    procedure ClearPlaylist;
    procedure SavePlaylist;
    procedure Play;
    procedure SetVolume(Value:byte=122);
    procedure SetPanning(Value:byte=122);
    procedure ShowOptions;
    procedure ShowAbout;
    procedure OpenFiles;
    procedure ToggleAlwaysOnTop;
    procedure ToggleEqualizer;
    procedure TogglePlaylist;
    procedure OpenFile(FileName:string);
    procedure SetDirectory(directory:string);
    procedure VolumeInc;
    procedure VolumeDec;
    procedure FastForward;
    procedure FastRewind;
    procedure OpenLocation;
    procedure PreviousTrack;
    procedure NextTrack;
    procedure Stop;
    procedure Pause;
    procedure StartOfList;
    procedure EndOfList;
    procedure StopWithFadout;
    function GetEqualizer:TWinampEqualizer;
    procedure SetEqualizer(Value:TWinampEqualizer);
  end;

implementation

resourcestring
   RC_WinampWindow     =    'Winamp v1.x';
   RC_WinampFormat     =    'You must have Winamp %d.%d or higher to execute this Api';
   RC_ErrorFinding     =    'Could not find winamp window';

{**************************************************}
procedure TJvWinampApi.ClearPlaylist;
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
     SendMessage(h,WM_WA_IPC,0,IPC_DELETE)
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}
function TJvWinampApi.GetBitRate: Integer;
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
     result := SendMessage(h,WM_WA_IPC,1,IPC_GETINFO)
   else
     result:=0;
end;
{**************************************************}
function TJvWinampApi.GetChannels: Integer;
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
     result := SendMessage(h,WM_WA_IPC,2,IPC_GETINFO)
   else
     result:=0;
end;
{**************************************************}
function TJvWinampApi.GetLength: TTime;
var
  h:THandle;
  i:Integer;
  tstamp:TTimeStamp;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   tstamp.Time:=0;
   tstamp.Date:=0;
   if h<>0 then
   begin
      i := SendMessage(h,WM_WA_IPC,1,IPC_GETOUTPUTTime);
      if i<>-1 then
           tstamp.Time:=i*1000;
   end;
   result:=TimeStampToDateTime(tstamp);
end;
{**************************************************}
function TJvWinampApi.GetMajorVersion: Integer;
var
  h:THandle;
  i:Integer;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
   begin
      i := SendMessage(h,WM_WA_IPC,0,IPC_GETVERSION);
      result := (i and $F000) div $1000;
   end
   else
       result:=-1;
end;
{**************************************************}
function TJvWinampApi.GetMinorVersion: Integer;
var
  h:THandle;
  i:Integer;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
   begin
      i := SendMessage(h,WM_WA_IPC,0,IPC_GETVERSION);
      if WinampMajorVersion=1 then
           result := ((i and $0F00) div (16*16*16))*10+(i and $000F)
      else
         result := StrToInt(inttohex(((i div 16) and $00FF),2));
   end
   else
       result:=-1;
end;
{**************************************************}
function TJvWinampApi.GetPos: Integer;
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
     result := SendMessage(h,WM_WA_IPC,0,IPC_GETLISTPOS)
   else
     result:=-1;
end;
{**************************************************}
function TJvWinampApi.GetSampleRate: Integer;
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
     result := SendMessage(h,WM_WA_IPC,0,IPC_GETINFO)
   else
     result:=0;
end;
{**************************************************}
function TJvWinampApi.GeTTime: TTime;
var
  h:THandle;
  i:Integer;
  tstamp:TTimeStamp;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   tstamp.Time:=0;
   tstamp.Date:=0;
   if h<>0 then
   begin
      i := SendMessage(h,WM_WA_IPC,0,IPC_GETOUTPUTTime);
      if i<>-1 then
           tstamp.Time:=i;
   end;
   result:=TimeStampToDateTime(tstamp);
end;
{**************************************************}
function TJvWinampApi.GetWinampPresent: boolean;
begin
   Result := FindWindow(PChar(RC_WinampWindow),nil)<>0;
end;
{**************************************************}
function TJvWinampApi.GetWStatus: TWStatus;
var
  h:THandle;
  i:Integer;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
   begin
      i := SendMessage(h,WM_WA_IPC,0,IPC_ISPLAYING);
      case i of
          0 : result:=wsStopped;
          1 : result:=wsPlaying;
          3 : result:=wsPaused;
          else result:=wsNotAvailable;
      end;
   end
   else
       result:=wsNotAvailable;
end;
{**************************************************}
procedure TJvWinampApi.Play;
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
     SendMessage(h,WM_WA_IPC,0,IPC_STARTPLAY)
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}
procedure TJvWinampApi.SavePlaylist;
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
   begin
      if WinampHigher(1,66) then
        SendMessage(h,WM_WA_IPC,0,IPC_WRITEPLAYLIST);
   end
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}
procedure TJvWinampApi.SetPos(const Value: integer);
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if (h<>0) and WinampHigher(2,0) then
     SendMessage(h,WM_WA_IPC,Value,IPC_SETPLAYLISTPOS);
end;
{**************************************************}
procedure TJvWinampApi.SetPanning(Value: byte);
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
   begin
      if WinampHigher(2,0) then
         SendMessage(h,WM_WA_IPC,Value,IPC_SETPANNING);
   end
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}
procedure TJvWinampApi.SeTTime(const Value: TTime);
var
  h:THandle;
  tstamp:TTimeStamp;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   tstamp:=DateTimeToTimeStamp(Value);
   if h<>0 then
      if WinampHigher(1,60) then
         SendMessage(h,WM_WA_IPC,tstamp.Time,IPC_JUMPTOTIME);
end;
{**************************************************}
procedure TJvWinampApi.SetVolume(Value: byte);
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
   begin
      if WinampHigher(2,0) then
         SendMessage(h,WM_WA_IPC,Value,IPC_SETVOLUME);
   end
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}
function TJvWinampApi.WinampHigher(Major, Minor:integer): boolean;
begin
   if (WinampMajorVersion>Major)or((Major=WinampMajorVersion)and(Minor<=WinampMinorVersion)) then
      result:=true
   else
   begin
      result:=false;
      if (not (csDesigning in ComponentState)) then
         raise EWinampError.Create(format(RC_WinampFormat,[Major,Minor]));
   end;
end;
{**************************************************}
function TJvWinampApi.GetListLength: Integer;
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if (h<>0)and(WinampHigher(2,0)) then
     result := SendMessage(h,WM_WA_IPC,0,IPC_GETLISTLENGTH)
   else
     result:=0;
end;
{**************************************************}
procedure TJvWinampApi.ShowAbout;
begin
   SendCommand(WINAMP_HELP_ABOUT)
end;
{**************************************************}
procedure TJvWinampApi.ShowOptions;
begin
   SendCommand(WINAMP_OPTIONS_PREFS)
end;
{**************************************************}
procedure TJvWinampApi.OpenFiles;
begin
   SendCommand(WINAMP_FILE_PLAY)
end;
{**************************************************}
procedure TJvWinampApi.ToggleAlwaysOnTop;
begin
   SendCommand(WINAMP_OPTIONS_AOT)
end;
{**************************************************}
procedure TJvWinampApi.ToggleEqualizer;
begin
   SendCommand(WINAMP_OPTIONS_EQ)
end;
{**************************************************}
procedure TJvWinampApi.TogglePlaylist;
begin
   SendCommand(WINAMP_OPTIONS_PLEDIT)
end;
{**************************************************}
procedure TJvWinampApi.OpenFile(FileName: string);
var
  cds:TCopyDataStruct;
  dat:array [0..255] of char;
  h:THandle;
begin
   cds.dwData:=IPC_PLAYFILE;
   strPCopy(dat,FileName);
   cds.lpData:=@dat;
   cds.cbData:=length(FileName);
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
     SendMessage(h,WM_CopyDATA,0,longint(@cds))
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}
procedure TJvWinampApi.SetDirectory(directory: string);
var
  cds:TCopyDataStruct;
  dat:array [0..255] of char;
  h:THandle;
begin
   cds.dwData:=IPC_CHDIR;
   strPCopy(dat,directory);
   cds.lpData:=@dat;
   cds.cbData:=length(directory);
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
     SendMessage(h,WM_CopyDATA,0,longint(@cds))
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}
procedure TJvWinampApi.VolumeDec;
begin
   SendCommand(WINAMP_VOLUMEDOWN)
end;
{**************************************************}
procedure TJvWinampApi.VolumeInc;
begin
   SendCommand(WINAMP_VOLUMEUP)
end;
{**************************************************}
procedure TJvWinampApi.FastForward;
begin
   SendCommand(WINAMP_FFWD5S)
end;
{**************************************************}
procedure TJvWinampApi.FastRewind;
begin
   SendCommand(WINAMP_REW5S)
end;
{**************************************************}
procedure TJvWinampApi.OpenLocation;
begin
   SendCommand(WINAMP_BUTTON2_CTRL)
end;
{**************************************************}
procedure TJvWinampApi.NextTrack;
begin
   SendCommand(WINAMP_BUTTON5)
end;
{**************************************************}
procedure TJvWinampApi.PreviousTrack;
begin
   SendCommand(WINAMP_BUTTON1)
end;
{**************************************************}
procedure TJvWinampApi.Pause;
begin
   SendCommand(WINAMP_BUTTON3)
end;
{**************************************************}
procedure TJvWinampApi.Stop;
begin
   SendCommand(WINAMP_BUTTON4)
end;
{**************************************************}
procedure TJvWinampApi.EndOfList;
begin
   SendCommand(WINAMP_BUTTON5_CTRL)
end;
{**************************************************}
procedure TJvWinampApi.StartOfList;
begin
   SendCommand(WINAMP_BUTTON1_CTRL)
end;
{**************************************************}
procedure TJvWinampApi.StopWithFadout;
begin
   SendCommand(WINAMP_BUTTON4_SHIFT);
end;
{**************************************************}
function TJvWinampApi.GetEqualizer: TWinampEqualizer;
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
   begin
      if WinampHigher(2,5) then
      begin
         result.bands[0]:=SendMessage(h,WM_WA_IPC,0,IPC_GETEQDATA);
         result.bands[1]:=SendMessage(h,WM_WA_IPC,1,IPC_GETEQDATA);
         result.bands[2]:=SendMessage(h,WM_WA_IPC,2,IPC_GETEQDATA);
         result.bands[3]:=SendMessage(h,WM_WA_IPC,3,IPC_GETEQDATA);
         result.bands[4]:=SendMessage(h,WM_WA_IPC,4,IPC_GETEQDATA);
         result.bands[5]:=SendMessage(h,WM_WA_IPC,5,IPC_GETEQDATA);
         result.bands[6]:=SendMessage(h,WM_WA_IPC,6,IPC_GETEQDATA);
         result.bands[7]:=SendMessage(h,WM_WA_IPC,7,IPC_GETEQDATA);
         result.bands[8]:=SendMessage(h,WM_WA_IPC,8,IPC_GETEQDATA);
         result.bands[9]:=SendMessage(h,WM_WA_IPC,9,IPC_GETEQDATA);
         result.PreAmp:=SendMessage(h,WM_WA_IPC,10,IPC_GETEQDATA);
         result.Enabled:=SendMessage(h,WM_WA_IPC,11,IPC_GETEQDATA)<>0;
         result.Autoload:=SendMessage(h,WM_WA_IPC,12,IPC_GETEQDATA)<>0;
      end;
   end
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}
procedure TJvWinampApi.SetEqualizer(Value: TWinampEqualizer);
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
   begin
     if WinampHigher(2,5) then
     begin
         SendMessage(h,WM_WA_IPC,0,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[0],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,1,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[1],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,2,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[2],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,3,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[3],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,4,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[4],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,5,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[5],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,6,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[6],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,7,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[7],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,8,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[8],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,9,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.bands[9],IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,10,IPC_GETEQDATA);
         SendMessage(h,WM_WA_IPC,Value.PreAmp,IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,11,IPC_GETEQDATA);
         if Value.Enabled then
            SendMessage(h,WM_WA_IPC,1,IPC_SETEQDATA)
         else
            SendMessage(h,WM_WA_IPC,0,IPC_SETEQDATA);

         SendMessage(h,WM_WA_IPC,12,IPC_GETEQDATA);
         if Value.Autoload then
            SendMessage(h,WM_WA_IPC,1,IPC_SETEQDATA)
         else
            SendMessage(h,WM_WA_IPC,0,IPC_SETEQDATA);
     end;
   end
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}
procedure TJvWinampApi.SendCommand(Value: integer);
var
  h:THandle;
begin
   h := FindWindow(PChar(RC_WinampWindow),nil);
   if h<>0 then
     SendMessage(h,WM_COMMAND,Value,0)
   else
     raise EWinampError.Create(RC_ErrorFinding);
end;
{**************************************************}

end.
